# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `ExperimenterGroup.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'GroupExperimenterMap' not in _M_omero.model.__dict__:
    _M_omero.model._t_GroupExperimenterMap = IcePy.declareClass('::omero::model::GroupExperimenterMap')
    _M_omero.model._t_GroupExperimenterMapPrx = IcePy.declareProxy('::omero::model::GroupExperimenterMap')

if 'Experimenter' not in _M_omero.model.__dict__:
    _M_omero.model._t_Experimenter = IcePy.declareClass('::omero::model::Experimenter')
    _M_omero.model._t_ExperimenterPrx = IcePy.declareProxy('::omero::model::Experimenter')

if 'ExperimenterGroupAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_ExperimenterGroupAnnotationLink = IcePy.declareClass('::omero::model::ExperimenterGroupAnnotationLink')
    _M_omero.model._t_ExperimenterGroupAnnotationLinkPrx = IcePy.declareProxy('::omero::model::ExperimenterGroupAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if '_t_ExperimenterGroupGroupExperimenterMapSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_ExperimenterGroupGroupExperimenterMapSeq = IcePy.defineSequence('::omero::model::ExperimenterGroupGroupExperimenterMapSeq', (), _M_omero.model._t_GroupExperimenterMap)

if '_t_ExperimenterGroupLinkedExperimenterSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_ExperimenterGroupLinkedExperimenterSeq = IcePy.defineSequence('::omero::model::ExperimenterGroupLinkedExperimenterSeq', (), _M_omero.model._t_Experimenter)

if '_t_ExperimenterGroupAnnotationLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_ExperimenterGroupAnnotationLinksSeq = IcePy.defineSequence('::omero::model::ExperimenterGroupAnnotationLinksSeq', (), _M_omero.model._t_ExperimenterGroupAnnotationLink)

if '_t_ExperimenterGroupLinkedAnnotationSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_ExperimenterGroupLinkedAnnotationSeq = IcePy.defineSequence('::omero::model::ExperimenterGroupLinkedAnnotationSeq', (), _M_omero.model._t_Annotation)

if 'ExperimenterGroup' not in _M_omero.model.__dict__:
    _M_omero.model.ExperimenterGroup = Ice.createTempClass()
    class ExperimenterGroup(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _name=None, _ldap=None, _groupExperimenterMapSeq=None, _groupExperimenterMapLoaded=False, _config=None, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None, _description=None):
            if Ice.getType(self) == _M_omero.model.ExperimenterGroup:
                raise RuntimeError('omero.model.ExperimenterGroup is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._name = _name
            self._ldap = _ldap
            self._groupExperimenterMapSeq = _groupExperimenterMapSeq
            self._groupExperimenterMapLoaded = _groupExperimenterMapLoaded
            self._config = _config
            self._annotationLinksSeq = _annotationLinksSeq
            self._annotationLinksLoaded = _annotationLinksLoaded
            self._annotationLinksCountPerOwner = _annotationLinksCountPerOwner
            self._description = _description

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::ExperimenterGroup', '::omero::model::IObject')

        def ice_id(self, current=None):
            return '::omero::model::ExperimenterGroup'

        def ice_staticId():
            return '::omero::model::ExperimenterGroup'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getName(self, current=None):
            pass

        def setName(self, theName, current=None):
            pass

        def getLdap(self, current=None):
            pass

        def setLdap(self, theLdap, current=None):
            pass

        def unloadGroupExperimenterMap(self, current=None):
            pass

        def sizeOfGroupExperimenterMap(self, current=None):
            pass

        def copyGroupExperimenterMap(self, current=None):
            pass

        def addGroupExperimenterMap(self, target, current=None):
            pass

        def addAllGroupExperimenterMapSet(self, targets, current=None):
            pass

        def removeGroupExperimenterMap(self, theTarget, current=None):
            pass

        def removeAllGroupExperimenterMapSet(self, targets, current=None):
            pass

        def clearGroupExperimenterMap(self, current=None):
            pass

        def reloadGroupExperimenterMap(self, toCopy, current=None):
            pass

        def linkExperimenter(self, addition, current=None):
            pass

        def addGroupExperimenterMapToBoth(self, link, bothSides, current=None):
            pass

        def findGroupExperimenterMap(self, removal, current=None):
            pass

        def unlinkExperimenter(self, removal, current=None):
            pass

        def removeGroupExperimenterMapFromBoth(self, link, bothSides, current=None):
            pass

        def linkedExperimenterList(self, current=None):
            pass

        def getConfigAsMap(self, current=None):
            pass

        def getConfig(self, current=None):
            pass

        def setConfig(self, theConfig, current=None):
            pass

        def unloadAnnotationLinks(self, current=None):
            pass

        def sizeOfAnnotationLinks(self, current=None):
            pass

        def copyAnnotationLinks(self, current=None):
            pass

        def addExperimenterGroupAnnotationLink(self, target, current=None):
            pass

        def addAllExperimenterGroupAnnotationLinkSet(self, targets, current=None):
            pass

        def removeExperimenterGroupAnnotationLink(self, theTarget, current=None):
            pass

        def removeAllExperimenterGroupAnnotationLinkSet(self, targets, current=None):
            pass

        def clearAnnotationLinks(self, current=None):
            pass

        def reloadAnnotationLinks(self, toCopy, current=None):
            pass

        def getAnnotationLinksCountPerOwner(self, current=None):
            pass

        def linkAnnotation(self, addition, current=None):
            pass

        def addExperimenterGroupAnnotationLinkToBoth(self, link, bothSides, current=None):
            pass

        def findExperimenterGroupAnnotationLink(self, removal, current=None):
            pass

        def unlinkAnnotation(self, removal, current=None):
            pass

        def removeExperimenterGroupAnnotationLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedAnnotationList(self, current=None):
            pass

        def getDescription(self, current=None):
            pass

        def setDescription(self, theDescription, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_ExperimenterGroup)

        __repr__ = __str__

    _M_omero.model.ExperimenterGroupPrx = Ice.createTempClass()
    class ExperimenterGroupPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.ExperimenterGroup._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.ExperimenterGroup._op_setVersion.end(self, _r)

        def getName(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getName.invoke(self, ((), _ctx))

        def begin_getName(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getName.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getName(self, _r):
            return _M_omero.model.ExperimenterGroup._op_getName.end(self, _r)

        def setName(self, theName, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_setName.invoke(self, ((theName, ), _ctx))

        def begin_setName(self, theName, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_setName.begin(self, ((theName, ), _response, _ex, _sent, _ctx))

        def end_setName(self, _r):
            return _M_omero.model.ExperimenterGroup._op_setName.end(self, _r)

        def getLdap(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getLdap.invoke(self, ((), _ctx))

        def begin_getLdap(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getLdap.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getLdap(self, _r):
            return _M_omero.model.ExperimenterGroup._op_getLdap.end(self, _r)

        def setLdap(self, theLdap, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_setLdap.invoke(self, ((theLdap, ), _ctx))

        def begin_setLdap(self, theLdap, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_setLdap.begin(self, ((theLdap, ), _response, _ex, _sent, _ctx))

        def end_setLdap(self, _r):
            return _M_omero.model.ExperimenterGroup._op_setLdap.end(self, _r)

        def unloadGroupExperimenterMap(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_unloadGroupExperimenterMap.invoke(self, ((), _ctx))

        def begin_unloadGroupExperimenterMap(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_unloadGroupExperimenterMap.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadGroupExperimenterMap(self, _r):
            return _M_omero.model.ExperimenterGroup._op_unloadGroupExperimenterMap.end(self, _r)

        def sizeOfGroupExperimenterMap(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_sizeOfGroupExperimenterMap.invoke(self, ((), _ctx))

        def begin_sizeOfGroupExperimenterMap(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_sizeOfGroupExperimenterMap.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfGroupExperimenterMap(self, _r):
            return _M_omero.model.ExperimenterGroup._op_sizeOfGroupExperimenterMap.end(self, _r)

        def copyGroupExperimenterMap(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_copyGroupExperimenterMap.invoke(self, ((), _ctx))

        def begin_copyGroupExperimenterMap(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_copyGroupExperimenterMap.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyGroupExperimenterMap(self, _r):
            return _M_omero.model.ExperimenterGroup._op_copyGroupExperimenterMap.end(self, _r)

        def addGroupExperimenterMap(self, target, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addGroupExperimenterMap.invoke(self, ((target, ), _ctx))

        def begin_addGroupExperimenterMap(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addGroupExperimenterMap.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addGroupExperimenterMap(self, _r):
            return _M_omero.model.ExperimenterGroup._op_addGroupExperimenterMap.end(self, _r)

        def addAllGroupExperimenterMapSet(self, targets, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addAllGroupExperimenterMapSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllGroupExperimenterMapSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addAllGroupExperimenterMapSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllGroupExperimenterMapSet(self, _r):
            return _M_omero.model.ExperimenterGroup._op_addAllGroupExperimenterMapSet.end(self, _r)

        def removeGroupExperimenterMap(self, theTarget, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeGroupExperimenterMap.invoke(self, ((theTarget, ), _ctx))

        def begin_removeGroupExperimenterMap(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeGroupExperimenterMap.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeGroupExperimenterMap(self, _r):
            return _M_omero.model.ExperimenterGroup._op_removeGroupExperimenterMap.end(self, _r)

        def removeAllGroupExperimenterMapSet(self, targets, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeAllGroupExperimenterMapSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllGroupExperimenterMapSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeAllGroupExperimenterMapSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllGroupExperimenterMapSet(self, _r):
            return _M_omero.model.ExperimenterGroup._op_removeAllGroupExperimenterMapSet.end(self, _r)

        def clearGroupExperimenterMap(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_clearGroupExperimenterMap.invoke(self, ((), _ctx))

        def begin_clearGroupExperimenterMap(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_clearGroupExperimenterMap.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearGroupExperimenterMap(self, _r):
            return _M_omero.model.ExperimenterGroup._op_clearGroupExperimenterMap.end(self, _r)

        def reloadGroupExperimenterMap(self, toCopy, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_reloadGroupExperimenterMap.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadGroupExperimenterMap(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_reloadGroupExperimenterMap.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadGroupExperimenterMap(self, _r):
            return _M_omero.model.ExperimenterGroup._op_reloadGroupExperimenterMap.end(self, _r)

        def linkExperimenter(self, addition, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_linkExperimenter.invoke(self, ((addition, ), _ctx))

        def begin_linkExperimenter(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_linkExperimenter.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkExperimenter(self, _r):
            return _M_omero.model.ExperimenterGroup._op_linkExperimenter.end(self, _r)

        def addGroupExperimenterMapToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addGroupExperimenterMapToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addGroupExperimenterMapToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addGroupExperimenterMapToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addGroupExperimenterMapToBoth(self, _r):
            return _M_omero.model.ExperimenterGroup._op_addGroupExperimenterMapToBoth.end(self, _r)

        def findGroupExperimenterMap(self, removal, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_findGroupExperimenterMap.invoke(self, ((removal, ), _ctx))

        def begin_findGroupExperimenterMap(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_findGroupExperimenterMap.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findGroupExperimenterMap(self, _r):
            return _M_omero.model.ExperimenterGroup._op_findGroupExperimenterMap.end(self, _r)

        def unlinkExperimenter(self, removal, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_unlinkExperimenter.invoke(self, ((removal, ), _ctx))

        def begin_unlinkExperimenter(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_unlinkExperimenter.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkExperimenter(self, _r):
            return _M_omero.model.ExperimenterGroup._op_unlinkExperimenter.end(self, _r)

        def removeGroupExperimenterMapFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeGroupExperimenterMapFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeGroupExperimenterMapFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeGroupExperimenterMapFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeGroupExperimenterMapFromBoth(self, _r):
            return _M_omero.model.ExperimenterGroup._op_removeGroupExperimenterMapFromBoth.end(self, _r)

        def linkedExperimenterList(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_linkedExperimenterList.invoke(self, ((), _ctx))

        def begin_linkedExperimenterList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_linkedExperimenterList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedExperimenterList(self, _r):
            return _M_omero.model.ExperimenterGroup._op_linkedExperimenterList.end(self, _r)

        def getConfigAsMap(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getConfigAsMap.invoke(self, ((), _ctx))

        def begin_getConfigAsMap(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getConfigAsMap.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getConfigAsMap(self, _r):
            return _M_omero.model.ExperimenterGroup._op_getConfigAsMap.end(self, _r)

        def getConfig(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getConfig.invoke(self, ((), _ctx))

        def begin_getConfig(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getConfig.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getConfig(self, _r):
            return _M_omero.model.ExperimenterGroup._op_getConfig.end(self, _r)

        def setConfig(self, theConfig, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_setConfig.invoke(self, ((theConfig, ), _ctx))

        def begin_setConfig(self, theConfig, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_setConfig.begin(self, ((theConfig, ), _response, _ex, _sent, _ctx))

        def end_setConfig(self, _r):
            return _M_omero.model.ExperimenterGroup._op_setConfig.end(self, _r)

        def unloadAnnotationLinks(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_unloadAnnotationLinks.invoke(self, ((), _ctx))

        def begin_unloadAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_unloadAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadAnnotationLinks(self, _r):
            return _M_omero.model.ExperimenterGroup._op_unloadAnnotationLinks.end(self, _r)

        def sizeOfAnnotationLinks(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_sizeOfAnnotationLinks.invoke(self, ((), _ctx))

        def begin_sizeOfAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_sizeOfAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfAnnotationLinks(self, _r):
            return _M_omero.model.ExperimenterGroup._op_sizeOfAnnotationLinks.end(self, _r)

        def copyAnnotationLinks(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_copyAnnotationLinks.invoke(self, ((), _ctx))

        def begin_copyAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_copyAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyAnnotationLinks(self, _r):
            return _M_omero.model.ExperimenterGroup._op_copyAnnotationLinks.end(self, _r)

        def addExperimenterGroupAnnotationLink(self, target, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addExperimenterGroupAnnotationLink.invoke(self, ((target, ), _ctx))

        def begin_addExperimenterGroupAnnotationLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addExperimenterGroupAnnotationLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addExperimenterGroupAnnotationLink(self, _r):
            return _M_omero.model.ExperimenterGroup._op_addExperimenterGroupAnnotationLink.end(self, _r)

        def addAllExperimenterGroupAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addAllExperimenterGroupAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllExperimenterGroupAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addAllExperimenterGroupAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllExperimenterGroupAnnotationLinkSet(self, _r):
            return _M_omero.model.ExperimenterGroup._op_addAllExperimenterGroupAnnotationLinkSet.end(self, _r)

        def removeExperimenterGroupAnnotationLink(self, theTarget, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeExperimenterGroupAnnotationLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeExperimenterGroupAnnotationLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeExperimenterGroupAnnotationLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeExperimenterGroupAnnotationLink(self, _r):
            return _M_omero.model.ExperimenterGroup._op_removeExperimenterGroupAnnotationLink.end(self, _r)

        def removeAllExperimenterGroupAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeAllExperimenterGroupAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllExperimenterGroupAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeAllExperimenterGroupAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllExperimenterGroupAnnotationLinkSet(self, _r):
            return _M_omero.model.ExperimenterGroup._op_removeAllExperimenterGroupAnnotationLinkSet.end(self, _r)

        def clearAnnotationLinks(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_clearAnnotationLinks.invoke(self, ((), _ctx))

        def begin_clearAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_clearAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearAnnotationLinks(self, _r):
            return _M_omero.model.ExperimenterGroup._op_clearAnnotationLinks.end(self, _r)

        def reloadAnnotationLinks(self, toCopy, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_reloadAnnotationLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadAnnotationLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_reloadAnnotationLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadAnnotationLinks(self, _r):
            return _M_omero.model.ExperimenterGroup._op_reloadAnnotationLinks.end(self, _r)

        def getAnnotationLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getAnnotationLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getAnnotationLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getAnnotationLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getAnnotationLinksCountPerOwner(self, _r):
            return _M_omero.model.ExperimenterGroup._op_getAnnotationLinksCountPerOwner.end(self, _r)

        def linkAnnotation(self, addition, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_linkAnnotation.invoke(self, ((addition, ), _ctx))

        def begin_linkAnnotation(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_linkAnnotation.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkAnnotation(self, _r):
            return _M_omero.model.ExperimenterGroup._op_linkAnnotation.end(self, _r)

        def addExperimenterGroupAnnotationLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addExperimenterGroupAnnotationLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addExperimenterGroupAnnotationLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_addExperimenterGroupAnnotationLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addExperimenterGroupAnnotationLinkToBoth(self, _r):
            return _M_omero.model.ExperimenterGroup._op_addExperimenterGroupAnnotationLinkToBoth.end(self, _r)

        def findExperimenterGroupAnnotationLink(self, removal, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_findExperimenterGroupAnnotationLink.invoke(self, ((removal, ), _ctx))

        def begin_findExperimenterGroupAnnotationLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_findExperimenterGroupAnnotationLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findExperimenterGroupAnnotationLink(self, _r):
            return _M_omero.model.ExperimenterGroup._op_findExperimenterGroupAnnotationLink.end(self, _r)

        def unlinkAnnotation(self, removal, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_unlinkAnnotation.invoke(self, ((removal, ), _ctx))

        def begin_unlinkAnnotation(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_unlinkAnnotation.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkAnnotation(self, _r):
            return _M_omero.model.ExperimenterGroup._op_unlinkAnnotation.end(self, _r)

        def removeExperimenterGroupAnnotationLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeExperimenterGroupAnnotationLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeExperimenterGroupAnnotationLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_removeExperimenterGroupAnnotationLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeExperimenterGroupAnnotationLinkFromBoth(self, _r):
            return _M_omero.model.ExperimenterGroup._op_removeExperimenterGroupAnnotationLinkFromBoth.end(self, _r)

        def linkedAnnotationList(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_linkedAnnotationList.invoke(self, ((), _ctx))

        def begin_linkedAnnotationList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_linkedAnnotationList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedAnnotationList(self, _r):
            return _M_omero.model.ExperimenterGroup._op_linkedAnnotationList.end(self, _r)

        def getDescription(self, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getDescription.invoke(self, ((), _ctx))

        def begin_getDescription(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_getDescription.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getDescription(self, _r):
            return _M_omero.model.ExperimenterGroup._op_getDescription.end(self, _r)

        def setDescription(self, theDescription, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_setDescription.invoke(self, ((theDescription, ), _ctx))

        def begin_setDescription(self, theDescription, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.ExperimenterGroup._op_setDescription.begin(self, ((theDescription, ), _response, _ex, _sent, _ctx))

        def end_setDescription(self, _r):
            return _M_omero.model.ExperimenterGroup._op_setDescription.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.ExperimenterGroupPrx.ice_checkedCast(proxy, '::omero::model::ExperimenterGroup', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.ExperimenterGroupPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::ExperimenterGroup'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_ExperimenterGroupPrx = IcePy.defineProxy('::omero::model::ExperimenterGroup', ExperimenterGroupPrx)

    _M_omero.model._t_ExperimenterGroup = IcePy.declareClass('::omero::model::ExperimenterGroup')

    _M_omero.model._t_ExperimenterGroup = IcePy.defineClass('::omero::model::ExperimenterGroup', ExperimenterGroup, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_name', (), _M_omero._t_RString, False, 0),
        ('_ldap', (), _M_omero._t_RBool, False, 0),
        ('_groupExperimenterMapSeq', (), _M_omero.model._t_ExperimenterGroupGroupExperimenterMapSeq, False, 0),
        ('_groupExperimenterMapLoaded', (), IcePy._t_bool, False, 0),
        ('_config', (), _M_omero.api._t_NamedValueList, False, 0),
        ('_annotationLinksSeq', (), _M_omero.model._t_ExperimenterGroupAnnotationLinksSeq, False, 0),
        ('_annotationLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_annotationLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0),
        ('_description', (), _M_omero._t_RString, False, 0)
    ))
    ExperimenterGroup._ice_type = _M_omero.model._t_ExperimenterGroup

    ExperimenterGroup._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    ExperimenterGroup._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    ExperimenterGroup._op_getName = IcePy.Operation('getName', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    ExperimenterGroup._op_setName = IcePy.Operation('setName', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    ExperimenterGroup._op_getLdap = IcePy.Operation('getLdap', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RBool, False, 0), ())
    ExperimenterGroup._op_setLdap = IcePy.Operation('setLdap', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RBool, False, 0),), (), None, ())
    ExperimenterGroup._op_unloadGroupExperimenterMap = IcePy.Operation('unloadGroupExperimenterMap', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    ExperimenterGroup._op_sizeOfGroupExperimenterMap = IcePy.Operation('sizeOfGroupExperimenterMap', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    ExperimenterGroup._op_copyGroupExperimenterMap = IcePy.Operation('copyGroupExperimenterMap', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_ExperimenterGroupGroupExperimenterMapSeq, False, 0), ())
    ExperimenterGroup._op_addGroupExperimenterMap = IcePy.Operation('addGroupExperimenterMap', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_GroupExperimenterMap, False, 0),), (), None, ())
    ExperimenterGroup._op_addAllGroupExperimenterMapSet = IcePy.Operation('addAllGroupExperimenterMapSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ExperimenterGroupGroupExperimenterMapSeq, False, 0),), (), None, ())
    ExperimenterGroup._op_removeGroupExperimenterMap = IcePy.Operation('removeGroupExperimenterMap', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_GroupExperimenterMap, False, 0),), (), None, ())
    ExperimenterGroup._op_removeAllGroupExperimenterMapSet = IcePy.Operation('removeAllGroupExperimenterMapSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ExperimenterGroupGroupExperimenterMapSeq, False, 0),), (), None, ())
    ExperimenterGroup._op_clearGroupExperimenterMap = IcePy.Operation('clearGroupExperimenterMap', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    ExperimenterGroup._op_reloadGroupExperimenterMap = IcePy.Operation('reloadGroupExperimenterMap', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ExperimenterGroup, False, 0),), (), None, ())
    ExperimenterGroup._op_linkExperimenter = IcePy.Operation('linkExperimenter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Experimenter, False, 0),), (), ((), _M_omero.model._t_GroupExperimenterMap, False, 0), ())
    ExperimenterGroup._op_addGroupExperimenterMapToBoth = IcePy.Operation('addGroupExperimenterMapToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_GroupExperimenterMap, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    ExperimenterGroup._op_findGroupExperimenterMap = IcePy.Operation('findGroupExperimenterMap', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Experimenter, False, 0),), (), ((), _M_omero.model._t_ExperimenterGroupGroupExperimenterMapSeq, False, 0), ())
    ExperimenterGroup._op_unlinkExperimenter = IcePy.Operation('unlinkExperimenter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Experimenter, False, 0),), (), None, ())
    ExperimenterGroup._op_removeGroupExperimenterMapFromBoth = IcePy.Operation('removeGroupExperimenterMapFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_GroupExperimenterMap, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    ExperimenterGroup._op_linkedExperimenterList = IcePy.Operation('linkedExperimenterList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_ExperimenterGroupLinkedExperimenterSeq, False, 0), ())
    ExperimenterGroup._op_getConfigAsMap = IcePy.Operation('getConfigAsMap', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_StringStringMap, False, 0), ())
    ExperimenterGroup._op_getConfig = IcePy.Operation('getConfig', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.api._t_NamedValueList, False, 0), ())
    ExperimenterGroup._op_setConfig = IcePy.Operation('setConfig', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.api._t_NamedValueList, False, 0),), (), None, ())
    ExperimenterGroup._op_unloadAnnotationLinks = IcePy.Operation('unloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    ExperimenterGroup._op_sizeOfAnnotationLinks = IcePy.Operation('sizeOfAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    ExperimenterGroup._op_copyAnnotationLinks = IcePy.Operation('copyAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_ExperimenterGroupAnnotationLinksSeq, False, 0), ())
    ExperimenterGroup._op_addExperimenterGroupAnnotationLink = IcePy.Operation('addExperimenterGroupAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ExperimenterGroupAnnotationLink, False, 0),), (), None, ())
    ExperimenterGroup._op_addAllExperimenterGroupAnnotationLinkSet = IcePy.Operation('addAllExperimenterGroupAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ExperimenterGroupAnnotationLinksSeq, False, 0),), (), None, ())
    ExperimenterGroup._op_removeExperimenterGroupAnnotationLink = IcePy.Operation('removeExperimenterGroupAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ExperimenterGroupAnnotationLink, False, 0),), (), None, ())
    ExperimenterGroup._op_removeAllExperimenterGroupAnnotationLinkSet = IcePy.Operation('removeAllExperimenterGroupAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ExperimenterGroupAnnotationLinksSeq, False, 0),), (), None, ())
    ExperimenterGroup._op_clearAnnotationLinks = IcePy.Operation('clearAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    ExperimenterGroup._op_reloadAnnotationLinks = IcePy.Operation('reloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ExperimenterGroup, False, 0),), (), None, ())
    ExperimenterGroup._op_getAnnotationLinksCountPerOwner = IcePy.Operation('getAnnotationLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    ExperimenterGroup._op_linkAnnotation = IcePy.Operation('linkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_ExperimenterGroupAnnotationLink, False, 0), ())
    ExperimenterGroup._op_addExperimenterGroupAnnotationLinkToBoth = IcePy.Operation('addExperimenterGroupAnnotationLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ExperimenterGroupAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    ExperimenterGroup._op_findExperimenterGroupAnnotationLink = IcePy.Operation('findExperimenterGroupAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_ExperimenterGroupAnnotationLinksSeq, False, 0), ())
    ExperimenterGroup._op_unlinkAnnotation = IcePy.Operation('unlinkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), None, ())
    ExperimenterGroup._op_removeExperimenterGroupAnnotationLinkFromBoth = IcePy.Operation('removeExperimenterGroupAnnotationLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ExperimenterGroupAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    ExperimenterGroup._op_linkedAnnotationList = IcePy.Operation('linkedAnnotationList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_ExperimenterGroupLinkedAnnotationSeq, False, 0), ())
    ExperimenterGroup._op_getDescription = IcePy.Operation('getDescription', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    ExperimenterGroup._op_setDescription = IcePy.Operation('setDescription', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())

    _M_omero.model.ExperimenterGroup = ExperimenterGroup
    del ExperimenterGroup

    _M_omero.model.ExperimenterGroupPrx = ExperimenterGroupPrx
    del ExperimenterGroupPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



# Generated by slice2py - DO NOT EDIT!
#

import Ice
Ice.updateModule("omero.model.enums")

# Modules:
import omero_model_AcquisitionMode_ice
import omero_model_AdminPrivilege_ice
import omero_model_ArcType_ice
import omero_model_Binning_ice
import omero_model_ChecksumAlgorithm_ice
import omero_model_ContrastMethod_ice
import omero_model_Correction_ice
import omero_model_DetectorType_ice
import omero_model_DimensionOrder_ice
import omero_model_EventType_ice
import omero_model_ExperimentType_ice
import omero_model_Family_ice
import omero_model_FilamentType_ice
import omero_model_FilterType_ice
import omero_model_Format_ice
import omero_model_Illumination_ice
import omero_model_Immersion_ice
import omero_model_JobStatus_ice
import omero_model_LaserMedium_ice
import omero_model_LaserType_ice
import omero_model_Medium_ice
import omero_model_MicrobeamManipulationType_ice
import omero_model_MicroscopeType_ice
import omero_model_PhotometricInterpretation_ice
import omero_model_PixelsType_ice
import omero_model_ProjectionAxis_ice
import omero_model_ProjectionType_ice
import omero_model_Pulse_ice
import omero_model_RenderingModel_ice
import omero_model_Units_ice

# Submodules:



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `System.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_ModelF_ice
import omero_SystemF_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.sys
__name__ = 'omero.sys'

if 'EventContext' not in _M_omero.sys.__dict__:
    _M_omero.sys.EventContext = Ice.createTempClass()
    class EventContext(Ice.Object):
        """
        Maps the ome.system.EventContext interface. Represents the
        information known by the server security system about the
        current user login.
        """
        def __init__(self, shareId=0, sessionId=0, sessionUuid='', userId=0, userName='', sudoerId=None, sudoerName=None, groupId=0, groupName='', isAdmin=False, adminPrivileges=None, eventId=0, eventType='', memberOfGroups=None, leaderOfGroups=None, groupPermissions=None):
            self.shareId = shareId
            self.sessionId = sessionId
            self.sessionUuid = sessionUuid
            self.userId = userId
            self.userName = userName
            self.sudoerId = sudoerId
            self.sudoerName = sudoerName
            self.groupId = groupId
            self.groupName = groupName
            self.isAdmin = isAdmin
            self.adminPrivileges = adminPrivileges
            self.eventId = eventId
            self.eventType = eventType
            self.memberOfGroups = memberOfGroups
            self.leaderOfGroups = leaderOfGroups
            self.groupPermissions = groupPermissions

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::sys::EventContext')

        def ice_id(self, current=None):
            return '::omero::sys::EventContext'

        def ice_staticId():
            return '::omero::sys::EventContext'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.sys._t_EventContext)

        __repr__ = __str__

    _M_omero.sys.EventContextPrx = Ice.createTempClass()
    class EventContextPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.sys.EventContextPrx.ice_checkedCast(proxy, '::omero::sys::EventContext', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.sys.EventContextPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::sys::EventContext'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.sys._t_EventContextPrx = IcePy.defineProxy('::omero::sys::EventContext', EventContextPrx)

    _M_omero.sys._t_EventContext = IcePy.declareClass('::omero::sys::EventContext')

    _M_omero.sys._t_EventContext = IcePy.defineClass('::omero::sys::EventContext', EventContext, -1, (), False, False, None, (), (
        ('shareId', (), IcePy._t_long, False, 0),
        ('sessionId', (), IcePy._t_long, False, 0),
        ('sessionUuid', (), IcePy._t_string, False, 0),
        ('userId', (), IcePy._t_long, False, 0),
        ('userName', (), IcePy._t_string, False, 0),
        ('sudoerId', (), _M_omero._t_RLong, False, 0),
        ('sudoerName', (), _M_omero._t_RString, False, 0),
        ('groupId', (), IcePy._t_long, False, 0),
        ('groupName', (), IcePy._t_string, False, 0),
        ('isAdmin', (), IcePy._t_bool, False, 0),
        ('adminPrivileges', (), _M_omero.api._t_StringSet, False, 0),
        ('eventId', (), IcePy._t_long, False, 0),
        ('eventType', (), IcePy._t_string, False, 0),
        ('memberOfGroups', (), _M_omero.sys._t_LongList, False, 0),
        ('leaderOfGroups', (), _M_omero.sys._t_LongList, False, 0),
        ('groupPermissions', (), _M_omero.model._t_Permissions, False, 0)
    ))
    EventContext._ice_type = _M_omero.sys._t_EventContext

    _M_omero.sys.EventContext = EventContext
    del EventContext

    _M_omero.sys.EventContextPrx = EventContextPrx
    del EventContextPrx

if 'Filter' not in _M_omero.sys.__dict__:
    _M_omero.sys.Filter = Ice.createTempClass()
    class Filter(Ice.Object):
        """
        Provides common filters which MAY be applied to a
        query. Check the documentation for the particular
        method for more information on how these values will
        be interpreted as well as default values if they
        are missing. In general they are intended to mean:
        unique        := similar to SQL's ""DISTINCT"" keyword
        ownerId       := (some) objects queried should belong
        to this user
        groupId       := (some) objects queried should belong
        to this group
        preferOwner   := true implies if if ownerId and groupId
        are both defined, use only ownerId
        offset/limit  := represent a page which should be loaded
        Note: servers may choose to impose a
        maximum limit.
        start/endTime := (some) objects queried should have been
        created and/or modified within time span.
        """
        def __init__(self, unique=None, ownerId=None, groupId=None, offset=None, limit=None, startTime=None, endTime=None):
            self.unique = unique
            self.ownerId = ownerId
            self.groupId = groupId
            self.offset = offset
            self.limit = limit
            self.startTime = startTime
            self.endTime = endTime

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::sys::Filter')

        def ice_id(self, current=None):
            return '::omero::sys::Filter'

        def ice_staticId():
            return '::omero::sys::Filter'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.sys._t_Filter)

        __repr__ = __str__

    _M_omero.sys.FilterPrx = Ice.createTempClass()
    class FilterPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.sys.FilterPrx.ice_checkedCast(proxy, '::omero::sys::Filter', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.sys.FilterPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::sys::Filter'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.sys._t_FilterPrx = IcePy.defineProxy('::omero::sys::Filter', FilterPrx)

    _M_omero.sys._t_Filter = IcePy.declareClass('::omero::sys::Filter')

    _M_omero.sys._t_Filter = IcePy.defineClass('::omero::sys::Filter', Filter, -1, (), False, False, None, (), (
        ('unique', (), _M_omero._t_RBool, False, 0),
        ('ownerId', (), _M_omero._t_RLong, False, 0),
        ('groupId', (), _M_omero._t_RLong, False, 0),
        ('offset', (), _M_omero._t_RInt, False, 0),
        ('limit', (), _M_omero._t_RInt, False, 0),
        ('startTime', (), _M_omero._t_RTime, False, 0),
        ('endTime', (), _M_omero._t_RTime, False, 0)
    ))
    Filter._ice_type = _M_omero.sys._t_Filter

    _M_omero.sys.Filter = Filter
    del Filter

    _M_omero.sys.FilterPrx = FilterPrx
    del FilterPrx

if 'Options' not in _M_omero.sys.__dict__:
    _M_omero.sys.Options = Ice.createTempClass()
    class Options(Ice.Object):
        """
        Similar to Filter, provides common options which MAY be
        applied on a given method. Check each interface's
        documentation for more details.
        leaves        := whether or not graph leaves (usually images)
        should be loaded
        orphan        := whether or not orphaned objects (e.g. datasets
        not in a project) should be loaded
        acquisition...:= whether or not acquisitionData (objectives, etc.)
        should be loaded
        cacheable      := whether or not the query is cacheable by Hibernate
        (use with caution: caching may be counterproductive)
        """
        def __init__(self, leaves=None, orphan=None, acquisitionData=None, cacheable=None):
            self.leaves = leaves
            self.orphan = orphan
            self.acquisitionData = acquisitionData
            self.cacheable = cacheable

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::sys::Options')

        def ice_id(self, current=None):
            return '::omero::sys::Options'

        def ice_staticId():
            return '::omero::sys::Options'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.sys._t_Options)

        __repr__ = __str__

    _M_omero.sys.OptionsPrx = Ice.createTempClass()
    class OptionsPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.sys.OptionsPrx.ice_checkedCast(proxy, '::omero::sys::Options', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.sys.OptionsPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::sys::Options'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.sys._t_OptionsPrx = IcePy.defineProxy('::omero::sys::Options', OptionsPrx)

    _M_omero.sys._t_Options = IcePy.declareClass('::omero::sys::Options')

    _M_omero.sys._t_Options = IcePy.defineClass('::omero::sys::Options', Options, -1, (), False, False, None, (), (
        ('leaves', (), _M_omero._t_RBool, False, 0),
        ('orphan', (), _M_omero._t_RBool, False, 0),
        ('acquisitionData', (), _M_omero._t_RBool, False, 0),
        ('cacheable', (), _M_omero._t_RBool, False, 0)
    ))
    Options._ice_type = _M_omero.sys._t_Options

    _M_omero.sys.Options = Options
    del Options

    _M_omero.sys.OptionsPrx = OptionsPrx
    del OptionsPrx

if 'Parameters' not in _M_omero.sys.__dict__:
    _M_omero.sys.Parameters = Ice.createTempClass()
    class Parameters(Ice.Object):
        """
        Holder for all the parameters which can be taken to a query.
        """
        def __init__(self, map=None, theFilter=None, theOptions=None):
            self.map = map
            self.theFilter = theFilter
            self.theOptions = theOptions

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::sys::Parameters')

        def ice_id(self, current=None):
            return '::omero::sys::Parameters'

        def ice_staticId():
            return '::omero::sys::Parameters'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.sys._t_Parameters)

        __repr__ = __str__

    _M_omero.sys.ParametersPrx = Ice.createTempClass()
    class ParametersPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.sys.ParametersPrx.ice_checkedCast(proxy, '::omero::sys::Parameters', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.sys.ParametersPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::sys::Parameters'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.sys._t_ParametersPrx = IcePy.defineProxy('::omero::sys::Parameters', ParametersPrx)

    _M_omero.sys._t_Parameters = IcePy.declareClass('::omero::sys::Parameters')

    _M_omero.sys._t_Parameters = IcePy.defineClass('::omero::sys::Parameters', Parameters, -1, (), False, False, None, (), (
        ('map', (), _M_omero.sys._t_ParamMap, False, 0),
        ('theFilter', (), _M_omero.sys._t_Filter, False, 0),
        ('theOptions', (), _M_omero.sys._t_Options, False, 0)
    ))
    Parameters._ice_type = _M_omero.sys._t_Parameters

    _M_omero.sys.Parameters = Parameters
    del Parameters

    _M_omero.sys.ParametersPrx = ParametersPrx
    del ParametersPrx

if 'Principal' not in _M_omero.sys.__dict__:
    _M_omero.sys.Principal = Ice.createTempClass()
    class Principal(Ice.Object):
        """
        Principal used for login, etc.
        """
        def __init__(self, name='', group='', eventType='', umask=None):
            self.name = name
            self.group = group
            self.eventType = eventType
            self.umask = umask

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::sys::Principal')

        def ice_id(self, current=None):
            return '::omero::sys::Principal'

        def ice_staticId():
            return '::omero::sys::Principal'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.sys._t_Principal)

        __repr__ = __str__

    _M_omero.sys.PrincipalPrx = Ice.createTempClass()
    class PrincipalPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.sys.PrincipalPrx.ice_checkedCast(proxy, '::omero::sys::Principal', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.sys.PrincipalPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::sys::Principal'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.sys._t_PrincipalPrx = IcePy.defineProxy('::omero::sys::Principal', PrincipalPrx)

    _M_omero.sys._t_Principal = IcePy.declareClass('::omero::sys::Principal')

    _M_omero.sys._t_Principal = IcePy.defineClass('::omero::sys::Principal', Principal, -1, (), False, False, None, (), (
        ('name', (), IcePy._t_string, False, 0),
        ('group', (), IcePy._t_string, False, 0),
        ('eventType', (), IcePy._t_string, False, 0),
        ('umask', (), _M_omero.model._t_Permissions, False, 0)
    ))
    Principal._ice_type = _M_omero.sys._t_Principal

    _M_omero.sys.Principal = Principal
    del Principal

    _M_omero.sys.PrincipalPrx = PrincipalPrx
    del PrincipalPrx

if 'Roles' not in _M_omero.sys.__dict__:
    _M_omero.sys.Roles = Ice.createTempClass()
    class Roles(Ice.Object):
        """
        Server-constants used for determining particular groups and
        users.
        """
        def __init__(self, rootId=0, rootName='', systemGroupId=0, systemGroupName='', userGroupId=0, userGroupName='', guestId=0, guestName='', guestGroupId=0, guestGroupName=''):
            self.rootId = rootId
            self.rootName = rootName
            self.systemGroupId = systemGroupId
            self.systemGroupName = systemGroupName
            self.userGroupId = userGroupId
            self.userGroupName = userGroupName
            self.guestId = guestId
            self.guestName = guestName
            self.guestGroupId = guestGroupId
            self.guestGroupName = guestGroupName

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::sys::Roles')

        def ice_id(self, current=None):
            return '::omero::sys::Roles'

        def ice_staticId():
            return '::omero::sys::Roles'
        ice_staticId = staticmethod(ice_staticId)

        def __str__(self):
            return IcePy.stringify(self, _M_omero.sys._t_Roles)

        __repr__ = __str__

    _M_omero.sys.RolesPrx = Ice.createTempClass()
    class RolesPrx(Ice.ObjectPrx):

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.sys.RolesPrx.ice_checkedCast(proxy, '::omero::sys::Roles', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.sys.RolesPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::sys::Roles'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.sys._t_RolesPrx = IcePy.defineProxy('::omero::sys::Roles', RolesPrx)

    _M_omero.sys._t_Roles = IcePy.defineClass('::omero::sys::Roles', Roles, -1, (), False, False, None, (), (
        ('rootId', (), IcePy._t_long, False, 0),
        ('rootName', (), IcePy._t_string, False, 0),
        ('systemGroupId', (), IcePy._t_long, False, 0),
        ('systemGroupName', (), IcePy._t_string, False, 0),
        ('userGroupId', (), IcePy._t_long, False, 0),
        ('userGroupName', (), IcePy._t_string, False, 0),
        ('guestId', (), IcePy._t_long, False, 0),
        ('guestName', (), IcePy._t_string, False, 0),
        ('guestGroupId', (), IcePy._t_long, False, 0),
        ('guestGroupName', (), IcePy._t_string, False, 0)
    ))
    Roles._ice_type = _M_omero.sys._t_Roles

    _M_omero.sys.Roles = Roles
    del Roles

    _M_omero.sys.RolesPrx = RolesPrx
    del RolesPrx

# End of module omero.sys

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright (C) 2008-2017 Glencoe Software, Inc. All Rights Reserved.
# Use is subject to license terms supplied in LICENSE.txt
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

"""
   Library for integration tests

"""
from __future__ import division
from __future__ import print_function

from builtins import str
from future.utils import native_str
from past.builtins import basestring
from builtins import range
from past.utils import old_div
from builtins import object
import os
import platform
import locale
import sys
import time
import weakref
import logging
import subprocess
import pytest
import uuid

import Ice
import Glacier2
import omero
import omero.gateway

from omero_version import omero_version
from collections import defaultdict

from omero.cmd import DoAll, State, ERR, OK, Chmod2, Chgrp2, Delete2
from omero.callbacks import CmdCallbackI
from omero.model import DatasetI, ImageI, ProjectI
from omero.model import Annotation, FileAnnotationI, TagAnnotationI
from omero.model import OriginalFileI
from omero.model import DimensionOrderI, PixelsI, PixelsTypeI
from omero.model import Experimenter, ExperimenterI
from omero.model import ExperimenterGroup, ExperimenterGroupI
from omero.model import PermissionsI
from omero.model import ChecksumAlgorithmI
from omero.model import NamedValue
from omero.model.enums import ChecksumAlgorithmSHA1160, DimensionOrderXYZCT
from omero.model.enums import PixelsTypeint8, PixelsTypeuint8, PixelsTypeint16
from omero.model.enums import PixelsTypeuint16, PixelsTypeint32
from omero.model.enums import PixelsTypeuint32, PixelsTypefloat
from omero.model.enums import PixelsTypedouble

from omero.rtypes import rbool, rstring, rlong, rtime, rint, unwrap
from omero.sys import ParametersI
from omero.util.temp_files import create_path
from omero_ext.path import path

import hashlib


class Clients(object):

    def __init__(self):
        self.__clients = set()

    def __del__(self):
        try:
            for client_ref in self.__clients:
                client = client_ref()
                if client:
                    client.__del__()
        finally:
            self.__clients = set()

    def add(self, client):
        self.__clients.add(weakref.ref(client))


class ITest(object):

    log = logging.getLogger("ITest")
    # Default permissions for the group created in setup_class
    # Can be overriden by test instances
    DEFAULT_PERMS = 'rw----'
    # If the new user created in setup_class should own their group
    # Can be overriden by test instances
    DEFAULT_GROUP_OWNER = False
    # If the new user created in setup_class should
    # be member of system group. This is to be
    # set to True if the new user should be Restricted Admin
    # Can be overriden by test instances
    DEFAULT_SYSTEM = False
    # If the new user created in setup_class is system group member
    # default value None makes the new user to Full Admin
    # For Restricted Admin with no privileges, set the value to ()
    # Can be overriden by test instances
    DEFAULT_PRIVILEGES = None

    @classmethod
    def setup_class(cls):

        cls.omero_dist = cls.omerodistdir()

        cls.__clients = Clients()

        # Create a root client
        p = Ice.createProperties(sys.argv)
        rootpass = p.getProperty("omero.rootpass")

        try:
            cls.root = omero.client()  # ok because adds self
            cls.__clients.add(cls.root)
            cls.root.setAgent("OMERO.py.root_test")
            cls.root.createSession("root", rootpass)
            cls.root.getSession().keepAlive(None)
        except Exception:
            cls.log.error("Could not initiate a root connection")
            raise

        cls.group = cls.new_group(perms=cls.DEFAULT_PERMS)
        cls.user = cls.new_user(group=cls.group,
                                owner=cls.DEFAULT_GROUP_OWNER,
                                system=cls.DEFAULT_SYSTEM,
                                privileges=cls.DEFAULT_PRIVILEGES)
        cls.client = omero.client()  # ok because adds self
        cls.__clients.add(cls.client)
        cls.client.setAgent("OMERO.py.test")
        cls.sf = cls.client.createSession(
            cls.user.omeName.val, cls.user.omeName.val)
        cls.ctx = cls.sf.getAdminService().getEventContext()
        cls.update = cls.sf.getUpdateService()
        cls.query = cls.sf.getQueryService()

    @classmethod
    def teardown_class(cls):
        cls.root.killSession()
        cls.root = None
        cls.__clients.__del__()

    def keep_root_alive(self):
        """
        Keeps root connection alive.
        """
        try:
            if self.root.sf is None:
                p = Ice.createProperties(sys.argv)
                rootpass = p.getProperty("omero.rootpass")
                self.root.createSession("root", rootpass)
            else:
                self.root.sf.keepAlive(None)
        except Exception:
            raise

    @classmethod
    def omerodistdir(cls):
        dist_dir = os.getenv('OMERODIR')
        if dist_dir:
            dist_dir = path(dist_dir)
        if dist_dir is not None:
            return dist_dir
        else:
            assert False, "OMERODIR is not set"

    def skip_if(self, config_key, condition, message=None):
        """Skip test if configuration does not meet condition"""
        config_service = self.root.sf.getConfigService()
        config_value = config_service.getConfigValue(config_key)
        if condition(config_value):
            pytest.skip(message or '%s:%s does not meet condition'
                        % (config_key, config_value))

    @classmethod
    def uuid(cls):
        return str(uuid.uuid4())

    @classmethod
    def login_args(cls, client=None):
        p = cls.client.ic.getProperties()
        host = p.getProperty("omero.host")
        port = p.getProperty("omero.port")
        if not client:
            key = cls.sf.ice_getIdentity().name
        else:
            key = client.sf.ice_getIdentity().name
        return ["-q", "-s", host, "-k", key, "-p", port]

    @classmethod
    def root_login_args(cls):
        p = cls.root.ic.getProperties()
        host = p.getProperty("omero.host")
        port = p.getProperty("omero.port")
        key = cls.root.sf.ice_getIdentity().name
        return ["-s", host, "-k", key, "-p", port]

    def tmpfile(self):
        return str(create_path())

    # Administrative methods
    @classmethod
    def new_group(cls, experimenters=None, perms=None,
                  config=None, gname=None):
        admin = cls.root.sf.getAdminService()
        if gname is None:
            gname = cls.uuid()
        group = ExperimenterGroupI()
        group.name = rstring(gname)
        group.ldap = rbool(False)
        group.config = config
        if perms:
            group.details.permissions = PermissionsI(perms)
        gid = admin.createGroup(group)
        group = admin.getGroup(gid)
        cls.add_experimenters(group, experimenters)
        return group

    @classmethod
    def add_experimenters(cls, group, experimenters):
        admin = cls.root.sf.getAdminService()
        if experimenters:
            for exp in experimenters:
                user, name = cls.user_and_name(exp)
                admin.addGroups(user, [group])

    def add_groups(self, experimenter, groups, owner=False):
        admin = self.root.sf.getAdminService()
        for group in groups:
            user, name = self.user_and_name(experimenter)
            admin.addGroups(user, [group])
            if owner:
                admin.setGroupOwner(group, user)

    def remove_experimenters(self, group, experimenters):
        admin = self.root.sf.getAdminService()
        if experimenters:
            for exp in experimenters:
                user, name = self.user_and_name(exp)
                admin.removeGroups(user, [group])

    def set_context(self, client, gid):
        rv = client.getStatefulServices()
        for prx in rv:
            prx.close()
        client.sf.setSecurityContext(ExperimenterGroupI(gid, False))

    # Import methods
    def import_image(self, filename=None, client=None, extra_args=None,
                     skip="all", **kwargs):
        """
        Imports the specified file.
        """
        if filename is None:
            raise Exception("No file specified")
        if client is None:
            client = self.client

        server = client.getProperty("omero.host")
        port = client.getProperty("omero.port")
        key = client.getSessionId()

        args = ["omero"]
        args.extend(["-s", server, "-k", key, "-p", port, "import"])
        if skip:
            args.extend(["--skip", skip])
        # Temporary fix to pass current tests by getting legacy output
        args.extend(["--output", "legacy"])
        args.extend(["--"])
        if extra_args:
            args.extend(extra_args)
        args.append(filename)

        popen = subprocess.Popen(args, cwd=str(self.omero_dist),
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE)
        out, err = popen.communicate()
        rc = popen.wait()
        if rc != 0:
            raise Exception("import failed: [%r] %s\n%s\n%s" % (
                args, rc, out, err))
        pix_ids = []
        for x in out.split(b"\n"):
            if x and x.find(b"Created") < 0 and x.find(b"#") < 0:
                try:    # if the line has an image ID...
                    image_id = str(int(x.strip()))
                    # Occasionally during tests an id is duplicated on stdout
                    if image_id not in pix_ids:
                        pix_ids.append(image_id)
                except:
                    pass
        return pix_ids

    def import_fake_file(self, images_count=1, name=None, client=None,
                         with_companion=False, skip="all", **kwargs):
        """
        Creates a fake file with an images_count of images, imports
        the file and then return the list of images.
        By default a single image is imported.
        """
        if client is None:
            client = self.client
        if name is None:
            name = "import_fake_file_%s" % images_count

        try:
            global_metadata = kwargs.pop("GlobalMetadata")
        except:
            global_metadata = None
        if global_metadata:
            with_companion = True

        append = ""

        # Only include images count if enabled; in the case of plates,
        # this will be unused
        if images_count >= 1:
            append = "series=%d%s" % (images_count, append)

        if kwargs:
            for k, v in list(kwargs.items()):
                append += "&%s=%s" % (k, v)

        query = client.sf.getQueryService()
        fake = create_path(name, "&%s.fake" % append)
        if with_companion:
            with open(fake.abspath() + ".ini", "w") as ini:
                if global_metadata:
                    ini.write("[GlobalMetadata]\n")
                    for k, v in list(global_metadata.items()):
                        ini.write("%s=%s\n" % (k, v))

        pixel_ids = self.import_image(
            filename=fake.abspath(), client=client, skip=skip, **kwargs)

        if images_count >= 1:
            assert images_count == len(pixel_ids)

        images = []
        for pix_id_str in pixel_ids:
            pixels = query.get("Pixels", int(pix_id_str))
            images.append(pixels.getImage())
        return images

    def import_plates(self, client=None, plates=1, plate_acqs=1, plate_cols=1,
                      plate_rows=1, fields=1, **kwargs):
        """
        Creates fake plates and imports them.
        """

        if client is None:
            client = self.client

        kwargs["plates"] = plates
        kwargs["plateAcqs"] = plate_acqs
        kwargs["plateCols"] = plate_cols
        kwargs["plateRows"] = plate_rows
        kwargs["fields"] = fields
        images = self.import_fake_file(images_count=0, client=client, **kwargs)
        images = [x.id.val for x in images]

        query = client.sf.getQueryService()
        plates = query.findAllByQuery((
            "select p from Plate p "
            "join fetch p.wells as w "
            "join fetch w.wellSamples as ws "
            "join fetch ws.image as i "
            "where i.id in (:ids)"),
            omero.sys.ParametersI().addIds(images))
        return plates

    def create_test_image(self, size_x=16, size_y=16, size_z=1, size_c=1,
                          size_t=1, session=None, name="testImage",
                          thumb=True):
        """
        Creates a test image of the required dimensions, where each pixel
        value is set to the value of x+y.
        Returns the image (ImageI)
        """
        from numpy import fromfunction, int16
        from omero.util import script_utils

        if session is None:
            session = self.root.sf
        rendering_engine = session.createRenderingEngine()
        query_service = session.getQueryService()
        pixels_service = session.getPixelsService()
        raw_pixel_store = session.createRawPixelsStore()
        container_service = session.getContainerService()

        def f1(x, y):
            return y

        def f2(x, y):
            return (x + y) // 2

        def f3(x, y):
            return x

        p_type = PixelsTypeint16
        # look up the PixelsType object from DB
        # omero::model::PixelsType
        pixels_type = query_service.findByQuery(
            "from PixelsType as p where p.value='%s'" % p_type, None)
        # if for example float32
        if pixels_type is None and p_type.startswith("float"):
            # omero::model::PixelsType
            v = PixelsTypefloat
            pixels_type = query_service.findByQuery(
                "from PixelsType as p where p.value='%s'" % v, None)
        if pixels_type is None:
            raise Exception("Unknown pixels type for: " % p_type)

        # code below here is very similar to combineImages.py
        # create an image in OMERO and populate the planes with numpy 2D arrays
        channel_list = list(range(1, size_c + 1))
        iid = pixels_service.createImage(size_x, size_y, size_z, size_t,
                                         channel_list, pixels_type,
                                         name, "description")
        image_id = iid.getValue()
        image = container_service.getImages("Image", [image_id], None)[0]

        pixels_id = image.getPrimaryPixels().getId().getValue()

        colour_map = {0: (0, 0, 255, 255), 1: (0, 255, 0, 255),
                      2: (255, 0, 0, 255), 3: (255, 0, 255, 255)}
        f_list = [f1, f2, f3]
        try:
            raw_pixel_store.setPixelsId(pixels_id, True)
            for the_c in range(size_c):
                min_value = 0
                max_value = 0
                f = f_list[the_c % len(f_list)]
                for the_z in range(size_z):
                    for the_t in range(size_t):
                        plane_2d = fromfunction(f, (size_y, size_x),
                                                dtype=int16)
                        script_utils.upload_plane(raw_pixel_store, plane_2d,
                                                  the_z, the_c, the_t)
                        min_value = min(min_value, plane_2d.min())
                        max_value = max(max_value, plane_2d.max())
                pixels_service.setChannelGlobalMinMax(pixels_id, the_c,
                                                      float(min_value),
                                                      float(max_value))
                rgba = None
                if the_c in colour_map:
                    rgba = colour_map[the_c]
                script_utils.reset_rendering_settings(rendering_engine,
                                                      pixels_id, the_c,
                                                      min_value, max_value,
                                                      rgba)
        finally:
            rendering_engine.close()
            raw_pixel_store.close()

        if thumb:
            # See #9070. Forcing a thumbnail creation
            tb = session.createThumbnailStore()
            try:
                s = tb.getThumbnailByLongestSideSet(rint(16), [pixels_id])
                assert s[pixels_id] != ''

            finally:
                tb.close()

        # Reloading image to prevent error on old pixels updateEvent
        return container_service.getImages("Image", [image_id], None)[0]

    def get_fileset(self, i, client=None):
        """
        Takes an image object and return a fileset object
        """
        if client is None:
            client = self.client
        query = client.sf.getQueryService()

        params = omero.sys.ParametersI()
        params.addIds([x.id.val for x in i])
        query1 = "select fs from Fileset fs "\
            "left outer join fetch fs.images as image "\
            "where image.id in (:ids)"
        rv = unwrap(query.projection(query1, params))
        return rv[0][0]

    def index(self, *objs):
        if objs:
            for obj in objs:
                self.root.sf.getUpdateService().indexObject(
                    obj, {"omero.group": "-1"})

    def wait_on_cmd(self, client, handle, loops=10, ms=500, passes=True):
        """
        Wait on an omero.cmd.HandlePrx to finish processing
        and then assert pass or fail. The callback is returned
        for accessing the Response and Status elements.
        """
        callback = omero.callbacks.CmdCallbackI(client, handle)
        callback.loop(loops, ms)  # throws on timeout
        rsp = callback.getResponse()
        is_ok = isinstance(rsp, OK)
        assert passes == is_ok, str(rsp)
        return callback

    @classmethod
    def new_user(cls, group=None, perms=None,
                 owner=False, system=False, uname=None,
                 email=None, privileges=None):
        """
        :owner: If user is to be an owner of the created group
        :system: If user is to be a system group member
        :privileges: If system group member is to have privileges
                     privileges=None gives all privileges (full admin)
                     privileges=() gives no privileges
        """

        if not cls.root:
            raise Exception("No root client. Cannot create user")

        admin_service = cls.root.getSession().getAdminService()
        if uname is None:
            uname = cls.uuid()

        # Create group if necessary
        if not group:
            g = cls.new_group(perms=perms)
            group = g.getName().getValue()
        else:
            g, group = cls.group_and_name(group)

        # Create user
        e = ExperimenterI()
        e.omeName = rstring(uname)
        e.firstName = rstring(uname)
        e.lastName = rstring(uname)
        e.ldap = rbool(False)
        e.email = rstring(email)
        list_of_groups = list()
        list_of_groups.append(admin_service.lookupGroup('user'))
        uid = admin_service.createExperimenterWithPassword(
            e, rstring(uname), g, list_of_groups)
        e = admin_service.lookupExperimenter(uname)
        if owner:
            admin_service.setGroupOwner(g, e)
        if privileges is not None:
            system = True
        if system:
            admin_service.addGroups(e, [ExperimenterGroupI(0, False)])

        if privileges is not None:
            to_set = []
            for p in privileges:
                privilege = omero.model.AdminPrivilegeI()
                privilege.setValue(rstring(p))
                to_set.append(privilege)
            admin_service.setAdminPrivileges(e, to_set)
        return admin_service.getExperimenter(uid)

    def new_client(self, group=None, user=None, perms=None,
                   owner=False, system=False, session=None,
                   password=None, email=None, privileges=None):
        """
        Like new_user() but returns an active client.

        Passing user= or session= will prevent self.new_user
        from being called, and instead the given user (by name
        or ExperimenterI) or session will be logged in.
        """
        props = self.root.getPropertyMap()
        if session is not None:
            if user is not None:
                self.log.warning("user= argument will be ignored: %s", user)
            session = unwrap(session)
            props["omero.user"] = session
            props["omero.pass"] = session
        else:
            if user is not None:
                user, name = self.user_and_name(user)
            else:
                user = self.new_user(group, perms, owner=owner,
                                     system=system, email=email,
                                     privileges=privileges)
            props["omero.user"] = user.omeName.val
            if password is not None:
                props["omero.pass"] = password
            else:
                props["omero.pass"] = user.omeName.val

        client = omero.client(props)
        self.__clients.add(client)
        client.setAgent("OMERO.py.new_client_test")
        client.createSession()
        return client

    def new_client_and_user(self, group=None, perms=None,
                            owner=False, system=False,
                            privileges=None):
        user = self.new_user(group, owner=owner, system=system, perms=perms,
                             privileges=privileges)
        client = self.new_client(
            group, user, perms=perms, owner=owner, system=system)
        return client, user

    def timeit(self, func, *args, **kwargs):
        start = time.time()
        rv = func(*args, **kwargs)
        stop = time.time()
        elapsed = stop - start
        return elapsed, rv

    @classmethod
    def group_and_name(cls, group):
        group = unwrap(group)
        admin = cls.root.sf.getAdminService()
        if isinstance(group, int):
            group = admin.getGroup(group)
            name = group.name.val
        elif isinstance(group, ExperimenterGroup):
            if group.isLoaded():
                name = group.name.val
                group = admin.lookupGroup(name)
            else:
                group = admin.getGroup(group.id.val)
                name = group.name.val
        elif isinstance(group, basestring):
            name = group
            group = admin.lookupGroup(name)
        elif isinstance(group, Experimenter):
            assert False,\
                "group is a user! Try adding group= to your method invocation"
        else:
            assert False, "Unknown type: %s=%s" % (type(group), group)

        return group, name

    @classmethod
    def user_and_name(cls, user):
        user = unwrap(user)
        admin = cls.root.sf.getAdminService()
        if isinstance(user, omero.clients.BaseClient):
            admin = user.sf.getAdminService()
            ec = admin.getEventContext()
            name = ec.userName
            user = admin.lookupExperimenter(name)
        elif isinstance(user, Experimenter):
            if user.isLoaded():
                name = user.omeName.val
                user = admin.lookupExperimenter(name)
            else:
                user = admin.getExperimenter(user.id.val)
                name = user.omeName.val
        elif isinstance(user, basestring):
            name = user
            user = admin.lookupExperimenter(name)
        elif isinstance(user, ExperimenterGroup):
            assert False,\
                "user is a group! Try adding user= to your method invocation"
        else:
            assert False, "Unknown type: %s=%s" % (type(user), user)

        return user, name

    #
    # Data methods
    #

    def missing_pyramid(self, client=None):
        """
        Creates and returns a pixels whose shape changes from
        1,1,4000,4000,1 to 4000,4000,1,1,1 making it a pyramid
        candidate but without the pyramid which is created on
        initial import in 4.3+. This simulates a big image that
        was imported in 4.2.
        """

        if client is None:
            client = self.client

        fake = create_path("missing_pyramid", "&sizeX=4000&sizeY=4000.fake")
        pixels_id = self.import_image(filename=fake.abspath(), client=client,
                                      skip="all")
        return pixels_id[0]

    def create_pixels(self, x=10, y=10, z=10, c=3, t=50,
                      pixels_type=PixelsTypeint8, client=None):
        """
        Creates an int8 pixel of the given size in the database.
        No data is written.
        """
        image = self.new_image()
        pixels = PixelsI()
        pixels.sizeX = rint(x)
        pixels.sizeY = rint(y)
        pixels.sizeZ = rint(z)
        pixels.sizeC = rint(c)
        pixels.sizeT = rint(t)
        pixels.sha1 = rstring("")
        pixels.pixelsType = PixelsTypeI()
        pixels.pixelsType.value = rstring(pixels_type)
        pixels.dimensionOrder = DimensionOrderI()
        pixels.dimensionOrder.value = rstring(DimensionOrderXYZCT)
        image.addPixels(pixels)

        if client is None:
            client = self.client
        update = client.sf.getUpdateService()
        image = update.saveAndReturnObject(image)
        return image.getPrimaryPixels()

    @classmethod
    def get_pixels_type(cls, pixels_type):
        if pixels_type in [PixelsTypeint8, PixelsTypeuint8]:
            return 1
        if pixels_type in [PixelsTypeint16, PixelsTypeuint16]:
            return 2
        if pixels_type in [PixelsTypeint32, PixelsTypeuint32,
                           PixelsTypefloat]:
            return 4
        if pixels_type in [PixelsTypedouble]:
            return 8
        raise Exception("Pixels Type %s not supported" % pixels_type)

    def write(self, pixels, rps, pixels_type=PixelsTypeint8):
        """
        Writes byte arrays consisting of [5] to as
        either planes or tiles depending on the pixel
        size.
        """
        p_type = self.get_pixels_type(pixels.pixelsType.getValue().getValue())
        if not rps.requiresPixelsPyramid():
            # By plane
            bytes_per_plane = pixels.sizeX.val * pixels.sizeY.val * p_type
            for z in range(pixels.sizeZ.val):
                for c in range(pixels.sizeC.val):
                    for t in range(pixels.sizeT.val):
                        rps.setPlane([5] * bytes_per_plane, z, c, t)
        else:
            # By tile
            w, h = rps.getTileSize()
            bytes_per_tile = w * h * p_type
            for z in range(pixels.sizeZ.val):
                for c in range(pixels.sizeC.val):
                    for t in range(pixels.sizeT.val):
                        for x in range(0, pixels.sizeX.val, w):
                            for y in range(0, pixels.sizeY.val, h):

                                changed = False
                                if x + w > pixels.sizeX.val:
                                    w = pixels.sizeX.val - x
                                    changed = True
                                if y + h > pixels.sizeY.val:
                                    h = pixels.sizeY.val - y
                                    changed = True
                                if changed:
                                    # Again assuming int8
                                    bytes_per_tile = w * h

                                args = ([5] * bytes_per_tile,
                                        z, c, t, x, y, w, h)
                                rps.setTile(*args)

    def login_attempt(self, name, t, pw="BAD", less=False):
        """
        Checks that login happens in less than or greater than
        the given time. By default, the password "BAD" is used,
        and the expectation is that login will take greather
        than the specified time since the password won't match.
        To check that logins happen more quickly, pass the
        correct password and less=True:

            login_attempt("user", 0.15, pw="REALVALUE", less=True)

        See integration.tickets4000 and 5000
        """
        c = omero.client()  # ok because followed by __del__
        try:
            t1 = time.time()
            try:
                c.createSession(name, pw)
                if pw == "BAD":
                    assert False, "Should not reach this point"
            except Glacier2.PermissionDeniedException:
                if pw != "BAD":
                    raise
            t2 = time.time()
            diff = (t2 - t1)
            if less:
                assert diff < t, "%s > %s" % (diff, t)
            else:
                assert diff > t, "%s < %s" % (diff, t)
        finally:
            c.__del__()

    def do_submit(self, request, client, test_should_pass=True,
                  omero_group=None):
        """
        Performs the request(s), waits on completion and checks that the
        result is not an error. The request can either be a single command
        or a list of commands. If the latter then the request list will be
        wrapped in a DoAll.
        """
        if isinstance(request, list):
            request = DoAll(request)

        sf = client.sf
        if omero_group is not None:
            prx = sf.submit(request, {'omero.group': native_str(omero_group)})
        else:
            prx = sf.submit(request)

        assert State.FAILURE not in prx.getStatus().flags

        cb = CmdCallbackI(client, prx)
        cb.loop(20, 500)

        assert prx.getResponse() is not None

        rsp = prx.getResponse()

        if test_should_pass:
            if isinstance(rsp, ERR):
                assert False, (
                    "Found ERR when test_should_pass==true: %s (%s) params=%s"
                    % (rsp.category, rsp.name, rsp.parameters))
            assert State.FAILURE not in prx.getStatus().flags
        else:
            if isinstance(rsp, OK):
                assert False, (
                    "Found OK when test_should_pass==false: %s" % rsp)
            assert State.FAILURE in prx.getStatus().flags

        return rsp

    # Object methods
    def new_object(self, classname, name=None, description=None):
        obj = classname()
        if not name:
            name = self.uuid()
        obj.setName(rstring(name))
        obj.setDescription(rstring(description))
        return obj

    def new_image(self, name=None, description=None, date=0):
        """
        Creates a new image object.
        If no name has been provided, a UUID string shall be used.
        :param name: The image name. If None, a UUID string will be used
        :param description: The image description
        :param date: The image acquisition date
        """
        img = self.new_object(ImageI, name=name, description=description)
        img.acquisitionDate = rtime(date)
        return img

    def new_project(self, name=None, description=None):
        """
        Creates a new project object.
        :param name: The project name. If None, a UUID string will be used
        :param description: The project description
        """
        return self.new_object(ProjectI, name=name, description=description)

    def new_dataset(self, name=None, description=None):
        """
        Creates a new dataset object.
        :param name: The dataset name. If None, a UUID string will be used
        :param description: The dataset description
        """
        return self.new_object(DatasetI, name=name, description=description)

    def new_tag(self, name=None, ns=None):
        """
        Creates a new tag object.
        :param name: The tag name. If None, a UUID string will be used
        :param ns: The namespace for the annotation. If None, do not set.
        """
        tag = self.new_object(TagAnnotationI, name=name)
        tag.setTextValue(rstring(name))
        if ns is not None:
            tag.setNs(rstring(ns))
        return tag

    def make_image(self, name=None, description=None, date=0, client=None):
        """
        Creates a new image instance and returns the persisted object.
        :param name: The image name. If None, a UUID string will be used
        :param description: The image description
        :param date: The image acquisition date
        :param client: The client to use to create the object
        """
        if client is None:
            client = self.client
        image = self.new_image(name=name, description=description, date=date)
        return client.sf.getUpdateService().saveAndReturnObject(image)

    def make_project(self, name=None, description=None, client=None):
        """
        Creates a new project instance and returns the persisted object.
        :param name: The project name. If None, a UUID string will be used
        :param description: The project description
        :param client: The client to use to create the object
        """
        if client is None:
            client = self.client
        project = self.new_project(name=name, description=description)
        return client.sf.getUpdateService().saveAndReturnObject(project)

    def make_dataset(self, name=None, description=None, client=None):
        """
        Creates a new dataset instance and returns the persisted object.
        :param name: The dataset name. If None, a UUID string will be used
        :param description: The dataset description
        :param client: The client to use to create the object
        """
        if client is None:
            client = self.client
        dataset = self.new_dataset(name=name, description=description)
        return client.sf.getUpdateService().saveAndReturnObject(dataset)

    def make_tag(self, name=None, client=None, ns=None):
        """
        Creates a new tag instance and returns the persisted object.
        :param name: The tag name. If None, a UUID string will be used
        :param client: The client to use to create the object
        :param ns: The namespace for the annotation. If None, do not set.
        """
        if client is None:
            client = self.client
        tag = self.new_tag(name=name, ns=ns)
        return client.sf.getUpdateService().saveAndReturnObject(tag)

    def create_datasets(self, count, base_name, client=None):
        """
        Creates a list of the given number of Dataset instances with names of
        the form "name [1]", "name [2]" etc and returns them in a list.
        :param count: The number of datasets to create
        :param description: The base name of the dataset
        :param client: The client to use to create the object
        """

        if client is None:
            client = self.client

        update = client.sf.getUpdateService()
        dsets = []
        for i in range(count):
            name = base_name + " [" + str(i + 1) + "]"
            dsets.append(self.new_dataset(name=name))
        return update.saveAndReturnArray(dsets)

    def make_file_annotation(self, name=None, binary=None, mimetype=None,
                             client=None, namespace=None):
        """
        Creates a file annotation with an original file.
        If no name has been provided, a UUID string shall be used.

        :param name: The name of the file
        :param binary: The binary data
        :param mimetype: The mimetype of the file.
        :param client: The client to use to create the object
        :param ns: The namespace for the annotation
        """

        if client is None:
            client = self.client
        update = client.sf.getUpdateService()

        # file
        if mimetype is None:
            mimetype = "application/octet-stream"
        if binary is None:
            binary = b"12345678910"
        if name is None:
            name = str(self.uuid())

        ofile = OriginalFileI()
        ofile.setName(rstring(name))
        ofile.setPath(rstring(str(self.uuid())))
        ofile.setSize(rlong(len(binary)))
        ofile.hasher = ChecksumAlgorithmI()
        ofile.hasher.value = rstring(ChecksumAlgorithmSHA1160)
        ofile.setMimetype(rstring(str(mimetype)))
        ofile = update.saveAndReturnObject(ofile)

        # save binary
        store = client.sf.createRawFileStore()
        try:
            store.setFileId(ofile.getId().getValue())
            store.write(binary, 0, len(binary))
            ofile = store.save()  # See ticket:1501
        finally:
            store.close()

        fa = FileAnnotationI()
        fa.setFile(ofile)
        if namespace is not None:
            fa.setNs(rstring(namespace))
        return update.saveAndReturnObject(fa)

    def link(self, obj1, obj2, client=None):
        """
        Links two linkable model entities together by creating an instance of
        the correct link entity (e.g. ProjectDatasetLinkI,
        ScreenAnnotationLinkI etc) and persisting it
        in the DB. Accepts client instance to allow calls to happen in correct
        user contexts.

        :param obj1: parent object
        :param obj2: child object
        :param client: The client to use to create the link
        """
        if client is None:
            client = self.client

        otype1 = obj1.ice_staticId().split("::")[-1]
        if isinstance(obj2, Annotation):
            otype2 = "Annotation"
        else:
            otype2 = obj2.ice_staticId().split("::")[-1]
        try:
            linktype = getattr(omero.model, "%s%sLinkI" % (otype1, otype2))
        except AttributeError:
            assert False, "Object type not supported."

        link = linktype()

        """check if object exist or not"""
        if obj1.id is None:
            link.setParent(obj1)
        else:
            link.setParent(obj1.proxy())
        if obj2.id is None:
            link.setChild(obj2)
        else:
            link.setChild(obj2.proxy())
        return client.sf.getUpdateService().saveAndReturnObject(link)

    def delete(self, objs):
        """
        Deletes model entities (ProjectI, DatasetI, ImageI, etc)
        by creating Delete2 commands and calling
        :func:`~test.ITest.do_submit`.

        :param obj: a list of objects to be deleted
        """
        to_delete = defaultdict(list)
        for obj in objs:
            t = obj.__class__.__name__
            to_delete[t].append(obj.id.val)
        command = Delete2(targetObjects=to_delete)
        self.do_submit(command, self.client)

    def change_group(self, obj, target, client=None):
        """
        Moves a list of model entities (ProjectI, DatasetI or ImageI)
        to the target group. Accepts a client instance to guarantee calls
        in correct user contexts. Creates Chgrp2 commands and calls
        :func:`~test.ITest.do_submit`.

        :param obj: a list of objects to be moved
        :param target: the ID of the target group
        :param client: user context
        """
        if client is None:
            client = self.client
        if isinstance(obj[0], ProjectI):
            t = "Project"
        elif isinstance(obj[0], DatasetI):
            t = "Dataset"
        elif isinstance(obj[0], ImageI):
            t = "Image"
        else:
            assert False, "Object type not supported."

        ids = [i.id.val for i in obj]
        command = Chgrp2(targetObjects={t: ids}, groupId=target)

        self.do_submit(command, client)

    def change_permissions(self, gid, perms, client=None):
        """
        Changes the permissions of an ExperimenterGroup object.
        Accepts a client instance to guarantee calls in correct user contexts.
        Creates Chmod2 commands and calls :func:`~test.ITest.do_submit`.

        :param gid: id of an ExperimenterGroup
        :param perms: permissions string
        :param client: user context
        """
        if client is None:
            client = self.client

        command = Chmod2(
            targetObjects={'ExperimenterGroup': [gid]}, permissions=perms)

        self.do_submit(command, client)

    def wait_for_pyramid(self, id, client=None):
        if client is None:
            client = self.client
        store = client.sf.createRawPixelsStore()
        not_ready = True
        count = 0
        elapse_time = 1  # time in seconds
        try:
            # Do not wait more than 60 seconds
            while not_ready and count < 60:
                try:
                    store.setPixelsId(id, True)
                    # No exception. The pyramid is now ready
                    not_ready = False
                except Exception as ex:
                    # try again in elapse_time
                    print(count, "Pyramid not ready:", ex.message)
                    time.sleep(elapse_time)
                    count = count + elapse_time
        finally:
            store.close()

    def import_pyramid(self, tmpdir, name=None, client=None, skip="all"):
        if name is None:
            name = "test&sizeX=4000&sizeY=4000.fake"
        fakefile = tmpdir.join(name)
        fakefile.write('')
        if client is None:
            client = self.client
        pixels = self.import_image(filename=str(fakefile), client=client,
                                   skip=skip)[0]
        id = int(float(pixels))
        assert id >= 0
        # wait for the pyramid to be generated
        self.wait_for_pyramid(id, client)
        query_service = client.sf.getQueryService()
        pix = query_service.findByQuery(
            "select p from Pixels p where p.id = :id",
            ParametersI().addId(id))
        return pix.image.id.val

    def calculate_sha1(self, data):
        h = hashlib.sha1()
        h.update(data)
        return h.hexdigest()


class ProjectionFixture(object):
    """
    Used to test the return values from:
        'select x.permissions from Object x'
    """

    def __init__(self, perms, writer, reader,
                 can_read,
                 can_annotate=False, can_delete=False,
                 can_edit=False, can_link=False,
                 can_chgrp=False, can_chown=False):
        self.perms = perms
        self.writer = writer
        self.reader = reader

        self.canRead = can_read
        self.canAnnotate = can_annotate
        self.canDelete = can_delete
        self.canEdit = can_edit
        self.canLink = can_link
        self.canChgrp = can_chgrp
        self.canChown = can_chown

    def get_name(self):
        name = self.perms
        for e in [self.writer, self.reader]:
            name += "-"
            if "admin" in e:
                name += "admin"
            elif "owner" in e:
                name += "owner"
            else:
                name += "member"
        return name


PF = ProjectionFixture
PFS = (
    # Private group as root
    PF("rw----", "system-admin", "system-admin", 1, 0, 1, 1, 0, 1, 1),
    PF("rw----", "system-admin", "group-owner", 1, 0, 1, 1, 0, 0, 1),
    PF("rw----", "system-admin", "member2", 0),
    # Private group as group-owner
    PF("rw----", "group-owner", "system-admin", 1, 0, 1, 1, 0, 1, 1),
    PF("rw----", "group-owner", "group-owner", 1, 0, 1, 1, 0, 0, 1),
    PF("rw----", "group-owner", "member2", 0),
    # Private group as member
    PF("rw----", "member1", "system-admin", 1, 0, 1, 1, 0, 1, 1),
    PF("rw----", "member1", "group-owner", 1, 0, 1, 1, 0, 0, 1),
    PF("rw----", "member1", "member2", 0),
    # Read-only group as root
    PF("rwr---", "system-admin", "system-admin", 1, 1, 1, 1, 1, 1, 1),
    PF("rwr---", "system-admin", "group-owner", 1, 1, 1, 1, 1, 0, 1),
    PF("rwr---", "system-admin", "member2", 1, 0, 0, 0, 0, 0, 0),
    # Read-only group as group-owner
    PF("rwr---", "group-owner", "system-admin", 1, 1, 1, 1, 1, 1, 1),
    PF("rwr---", "group-owner", "group-owner", 1, 1, 1, 1, 1, 0, 1),
    PF("rwr---", "group-owner", "member2", 1, 0, 0, 0, 0, 0, 0),
    # Read-only group as member
    PF("rwr---", "member1", "system-admin", 1, 1, 1, 1, 1, 1, 1),
    PF("rwr---", "member1", "group-owner", 1, 1, 1, 1, 1, 0, 1),
    PF("rwr---", "member1", "member2", 1, 0, 0, 0, 0, 0, 0),
    # Read-annotate group as root
    PF("rwra--", "system-admin", "system-admin", 1, 1, 1, 1, 1, 1, 1),
    PF("rwra--", "system-admin", "group-owner", 1, 1, 1, 1, 1, 0, 1),
    PF("rwra--", "system-admin", "member2", 1, 1, 0, 0, 0, 0, 0),
    # Read-annotate group as group-owner
    PF("rwra--", "group-owner", "system-admin", 1, 1, 1, 1, 1, 1, 1),
    PF("rwra--", "group-owner", "group-owner", 1, 1, 1, 1, 1, 0, 1),
    PF("rwra--", "group-owner", "member2", 1, 1, 0, 0, 0, 0, 0),
    # Read-annotate group as member
    PF("rwra--", "member1", "system-admin", 1, 1, 1, 1, 1, 1, 1),
    PF("rwra--", "member1", "group-owner", 1, 1, 1, 1, 1, 0, 1),
    PF("rwra--", "member1", "member2", 1, 1, 0, 0, 0, 0, 0),
    # Read-write group as root
    PF("rwrw--", "system-admin", "system-admin", 1, 1, 1, 1, 1, 1, 1),
    PF("rwrw--", "system-admin", "group-owner", 1, 1, 1, 1, 1, 0, 1),
    PF("rwrw--", "system-admin", "member2", 1, 1, 1, 1, 1, 0, 0),
    # Read-write group as group-owner
    PF("rwrw--", "group-owner", "system-admin", 1, 1, 1, 1, 1, 1, 1),
    PF("rwrw--", "group-owner", "group-owner", 1, 1, 1, 1, 1, 0, 1),
    PF("rwrw--", "group-owner", "member2", 1, 1, 1, 1, 1, 0, 0),
    # Read-write group as member
    PF("rwrw--", "member1", "system-admin", 1, 1, 1, 1, 1, 1, 1),
    PF("rwrw--", "member1", "group-owner", 1, 1, 1, 1, 1, 0, 1),
    PF("rwrw--", "member1", "member2", 1, 1, 1, 1, 1, 0, 0),
)


class AbstractRepoTest(ITest):

    def setup_method(self, method):
        self.unique_dir = self.test_dir()

    def test_dir(self, client=None):
        if client is None:
            client = self.client
        mrepo = self.get_managed_repo(client=client)
        user_dir = self.user_dir(client=client)
        unique_dir = user_dir + "/" + self.uuid()  # ok
        mrepo.makeDir(unique_dir, True)
        return unique_dir

    def user_dir(self, client=None):
        if client is None:
            client = self.client
        ec = client.sf.getAdminService().getEventContext()
        return "%s_%s" % (ec.userName, ec.userId)

    def all(self, client):
        ctx = dict(client.getImplicitContext().getContext())
        ctx["omero.group"] = "-1"
        return ctx

    def get_managed_repo(self, client=None):
        if client is None:
            client = self.client
        mrepo = client.getManagedRepository()
        assert mrepo
        return mrepo

    def create_file(self, mrepo1, filename):
        rfs = mrepo1.file(filename, "rw")
        try:
            rfs.write(b"hi", 0, 2)
            ofile = rfs.save()
            return ofile
        finally:
            rfs.close()

    def raw(self, command, args, client=None):
        if client is None:
            client = self.client
        mrepo = self.get_managed_repo(self.client)
        obj = mrepo.root()
        sha = obj.hash.val
        raw_access = omero.grid.RawAccessRequest()
        raw_access.repoUuid = sha
        raw_access.command = command
        raw_access.args = args
        handle = client.sf.submit(raw_access)
        return CmdCallbackI(client, handle)

    def create_test_dir(self):
        folder = create_path(folder=True)
        (old_div(folder, "a.fake")).touch()
        (old_div(folder, "b.fake")).touch()
        return folder

    def create_fileset(self, folder):
        fileset = omero.model.FilesetI()
        for f in folder.files():
            entry = omero.model.FilesetEntryI()
            entry.setClientPath(rstring(str(f.abspath())))
            fileset.addFilesetEntry(entry)

        # Fill version info
        system, node, release, version, machine, processor = platform.uname()

        client_version_info = [
            NamedValue('omero.version', omero_version),
            NamedValue('os.name', system),
            NamedValue('os.version', release),
            NamedValue('os.architecture', machine)
        ]
        try:
            client_version_info.append(
                NamedValue('locale', locale.getdefaultlocale()[0]))
        except:
            pass

        upload = omero.model.UploadJobI()
        upload.setVersionInfo(client_version_info)
        fileset.linkJob(upload)
        return fileset

    def create_settings(self):
        settings = omero.grid.ImportSettings()
        settings.doThumbnails = rbool(True)
        settings.noStatsInfo = rbool(False)
        settings.userSpecifiedTarget = None
        settings.userSpecifiedName = None
        settings.userSpecifiedDescription = None
        settings.userSpecifiedAnnotationList = None
        settings.userSpecifiedPixels = None
        settings.checksumAlgorithm = ChecksumAlgorithmI()
        s = rstring(ChecksumAlgorithmSHA1160)
        settings.checksumAlgorithm.value = s
        return settings

    def upload_folder(self, proc, folder):
        ret_val = []
        for i, fobj in enumerate(folder.files()):  # Assuming same order
            rfs = proc.getUploader(i)
            try:
                f = fobj.open()
                try:
                    offset = 0
                    block = []
                    rfs.write(block, offset, len(block))  # Touch
                    while True:
                        block = f.read(1000 * 1000)
                        if not block:
                            break
                        rfs.write(block, offset, len(block))
                        offset += len(block)
                    ret_val.append(self.client.sha1(fobj.abspath()))
                finally:
                    f.close()
            finally:
                rfs.close()
        return ret_val

    def full_import(self, client):
        """
        Re-usable method for a basic import
        """
        mrepo = self.get_managed_repo(client)
        folder = self.create_test_dir()
        fileset = self.create_fileset(folder)
        settings = self.create_settings()

        proc = mrepo.importFileset(fileset, settings)
        try:
            return self.assert_import(client, proc, folder)
        finally:
            proc.close()

    # Assert methods
    def assert_import(self, client, proc, folder):
        hashes = self.upload_folder(proc, folder)
        handle = proc.verifyUpload(hashes)
        cb = CmdCallbackI(client, handle)
        rsp = self.assert_passes(cb)
        assert 1 == len(rsp.pixels)
        return rsp

    def assert_write(self, mrepo2, filename, ofile):
        def _write(rfs):
            try:
                rfs.write(b"bye", 0, 3)
                assert b"bye" == rfs.read(0, 3)
                # Resetting for other expectations
                rfs.truncate(2)
                rfs.write(b"hi", 0, 2)
                assert b"hi" == rfs.read(0, 2)
            finally:
                rfs.close()

        # TODO: fileById is always "r"
        # rfs = mrepo2.fileById(ofile.id.val)
        # _write(rfs)

        rfs = mrepo2.file(filename, "rw")
        _write(rfs)

    def assert_no_write(self, mrepo2, filename, ofile):
        def _nowrite(rfs):
            try:
                pytest.raises(omero.SecurityViolation,
                              rfs.write, b"bye", 0, 3)
                assert b"hi" == rfs.read(0, 2)
            finally:
                rfs.close()

        rfs = mrepo2.fileById(ofile.id.val)
        _nowrite(rfs)

        rfs = mrepo2.file(filename, "r")
        _nowrite(rfs)

        # Can't even acquire a writeable-rfs.
        pytest.raises(omero.SecurityViolation,
                      mrepo2.file, filename, "rw")

    def assert_dir_write(self, mrepo2, dirname):
        self.create_file(mrepo2, dirname + "/file2.txt")

    def assert_no_dir_write(self, mrepo2, dirname):
        # Also check that it's not possible to write
        # in someone else's directory.
        pytest.raises(omero.SecurityViolation,
                      self.create_file, mrepo2, dirname + "/file2.txt")

    def assert_no_read(self, mrepo2, filename, ofile):
        pytest.raises(omero.SecurityViolation,
                      mrepo2.fileById, ofile.id.val)
        pytest.raises(omero.SecurityViolation,
                      mrepo2.file, filename, "r")

    def assert_read(self, mrepo2, filename, ofile, ctx=None):
        def _read(rfs):
            try:
                assert b"hi" == rfs.read(0, 2)
            finally:
                rfs.close()

        rfs = mrepo2.fileById(ofile.id.val, ctx)
        _read(rfs)

        rfs = mrepo2.file(filename, "r", ctx)
        _read(rfs)

    def assert_listings(self, mrepo1, unique_dir):
        assert [unique_dir + "/b"] == mrepo1.list(unique_dir + "/")
        assert [unique_dir + "/b/c"] == mrepo1.list(unique_dir + "/b/")
        assert [
            unique_dir + "/b/c/file.txt"] == mrepo1.list(unique_dir + "/b/c/")

    def assert_passes(self, cb, loops=10, wait=500):
        cb.loop(loops, wait)
        rsp = cb.getResponse()
        if isinstance(rsp, omero.cmd.ERR):
            raise Exception(rsp)
        return rsp

    def assert_error(self, cb, loops=10, wait=500):
        cb.loop(loops, wait)
        rsp = cb.getResponse()
        assert isinstance(rsp, omero.cmd.ERR)



"""
   /*
   **   Generated by blitz/resources/templates/combined.vm
   **
   **   Copyright 2007, 2008 Glencoe Software, Inc. All rights reserved.
   **   Use is subject to license terms supplied in LICENSE.txt
   **
   */
"""
try:
   unicode
except NameError:
   # Python 3: "unicode" is built-in
   unicode = str
import Ice
import IceImport
import omero
IceImport.load("omero_model_DetailsI")
IceImport.load("omero_model_ChannelBinding_ice")
from omero.rtypes import rlong
from collections import namedtuple
_omero = Ice.openModule("omero")
_omero_model = Ice.openModule("omero.model")
__name__ = "omero.model"
class ChannelBindingI(_omero_model.ChannelBinding):

      # Property Metadata
      _field_info_data = namedtuple("FieldData", ["wrapper", "nullable"])
      _field_info_type = namedtuple("FieldInfo", [
          "renderingDef",
          "family",
          "coefficient",
          "inputStart",
          "inputEnd",
          "active",
          "noiseReduction",
          "red",
          "green",
          "blue",
          "alpha",
          "lookupTable",
          "spatialDomainEnhancement",
          "details",
      ])
      _field_info = _field_info_type(
          renderingDef=_field_info_data(wrapper=omero.proxy_to_instance, nullable=False),
          family=_field_info_data(wrapper=omero.proxy_to_instance, nullable=False),
          coefficient=_field_info_data(wrapper=omero.rtypes.rdouble, nullable=False),
          inputStart=_field_info_data(wrapper=omero.rtypes.rdouble, nullable=False),
          inputEnd=_field_info_data(wrapper=omero.rtypes.rdouble, nullable=False),
          active=_field_info_data(wrapper=omero.rtypes.rbool, nullable=False),
          noiseReduction=_field_info_data(wrapper=omero.rtypes.rbool, nullable=False),
          red=_field_info_data(wrapper=omero.rtypes.rint, nullable=False),
          green=_field_info_data(wrapper=omero.rtypes.rint, nullable=False),
          blue=_field_info_data(wrapper=omero.rtypes.rint, nullable=False),
          alpha=_field_info_data(wrapper=omero.rtypes.rint, nullable=False),
          lookupTable=_field_info_data(wrapper=omero.rtypes.rstring, nullable=True),
          spatialDomainEnhancement=_field_info_data(wrapper=omero.proxy_to_instance, nullable=False),
          details=_field_info_data(wrapper=omero.proxy_to_instance, nullable=True),
      )  # end _field_info
      RENDERINGDEF =  "ome.model.display.ChannelBinding_renderingDef"
      FAMILY =  "ome.model.display.ChannelBinding_family"
      COEFFICIENT =  "ome.model.display.ChannelBinding_coefficient"
      INPUTSTART =  "ome.model.display.ChannelBinding_inputStart"
      INPUTEND =  "ome.model.display.ChannelBinding_inputEnd"
      ACTIVE =  "ome.model.display.ChannelBinding_active"
      NOISEREDUCTION =  "ome.model.display.ChannelBinding_noiseReduction"
      RED =  "ome.model.display.ChannelBinding_red"
      GREEN =  "ome.model.display.ChannelBinding_green"
      BLUE =  "ome.model.display.ChannelBinding_blue"
      ALPHA =  "ome.model.display.ChannelBinding_alpha"
      LOOKUPTABLE =  "ome.model.display.ChannelBinding_lookupTable"
      SPATIALDOMAINENHANCEMENT =  "ome.model.display.ChannelBinding_spatialDomainEnhancement"
      DETAILS =  "ome.model.display.ChannelBinding_details"
      def errorIfUnloaded(self):
          if not self._loaded:
              raise _omero.UnloadedEntityException("Object unloaded:"+str(self))

      def throwNullCollectionException(self,propertyName):
          raise _omero.UnloadedEntityException(""+
          "Error updating collection:" + propertyName +"\n"+
          "Collection is currently null. This can be seen\n" +
          "by testing \""+ propertyName +"Loaded\". This implies\n"+
          "that this collection was unloaded. Please refresh this object\n"+
          "in order to update this collection.\n")

      def _toggleCollectionsLoaded(self, load):
          if load:
              self._spatialDomainEnhancementSeq = []
              self._spatialDomainEnhancementLoaded = True;
          else:
              self._spatialDomainEnhancementSeq = []
              self._spatialDomainEnhancementLoaded = False;

          pass

      def __init__(self, id=None, loaded=None):
          super(ChannelBindingI, self).__init__()
          if id is not None and isinstance(id, (str, unicode)) and ":" in id:
              parts = id.split(":")
              if len(parts) != 2:
                  raise Exception("Invalid proxy string: %s", id)
              if parts[0] != self.__class__.__name__ and \
                 parts[0]+"I" != self.__class__.__name__:
                  raise Exception("Proxy class mismatch: %s<>%s" %
                  (self.__class__.__name__, parts[0]))
              self._id = rlong(parts[1])
              if loaded is None:
                  # If no loadedness was requested with
                  # a proxy string, then assume False.
                  loaded = False
          else:
              # Relying on omero.rtypes.rlong's error-handling
              self._id = rlong(id)
              if loaded is None:
                  loaded = True  # Assume true as previously
          self._loaded = loaded
          if self._loaded:
             self._details = _omero_model.DetailsI()
             self._toggleCollectionsLoaded(True)

      def unload(self, current = None):
          self._loaded = False
          self.unloadRenderingDef( )
          self.unloadFamily( )
          self.unloadCoefficient( )
          self.unloadInputStart( )
          self.unloadInputEnd( )
          self.unloadActive( )
          self.unloadNoiseReduction( )
          self.unloadRed( )
          self.unloadGreen( )
          self.unloadBlue( )
          self.unloadAlpha( )
          self.unloadLookupTable( )
          self.unloadSpatialDomainEnhancement( )
          self.unloadDetails( )

      def isLoaded(self, current = None):
          return self._loaded
      def unloadCollections(self, current = None):
          self._toggleCollectionsLoaded( False )
      def isGlobal(self, current = None):
          return  False ;
      def isMutable(self, current = None):
          return  True ;
      def isAnnotated(self, current = None):
          return  False ;
      def isLink(self, current = None):
          return  False ;
      def shallowCopy(self, current = None):
            if not self._loaded: return self.proxy()
            copy = ChannelBindingI()
            copy._id = self._id;
            copy._version = self._version;
            copy._details = None  # Unloading for the moment.
            raise omero.ClientError("NYI")
      def proxy(self, current = None):
          if self._id is None: raise omero.ClientError("Proxies require an id")
          return ChannelBindingI( self._id.getValue(), False )

      def getDetails(self, current = None):
          self.errorIfUnloaded()
          return self._details

      def unloadDetails(self, current = None):
          self._details = None

      def getId(self, current = None):
          return self._id

      def setId(self, _id, current = None):
          self._id = _id

      def checkUnloadedProperty(self, value, loadedField):
          if value == None:
              self.__dict__[loadedField] = False
          else:
              self.__dict__[loadedField] = True

      def getVersion(self, current = None):
          self.errorIfUnloaded()
          return self._version

      def setVersion(self, version, current = None):
          self.errorIfUnloaded()
          self._version = version

      def unloadRenderingDef(self, ):
          self._renderingDefLoaded = False
          self._renderingDef = None;

      def getRenderingDef(self, current = None):
          self.errorIfUnloaded()
          return self._renderingDef

      def setRenderingDef(self, _renderingDef, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.renderingDef.wrapper is not None:
              if _renderingDef is not None:
                  _renderingDef = self._field_info.renderingDef.wrapper(_renderingDef)
          self._renderingDef = _renderingDef
          pass

      def unloadFamily(self, ):
          self._familyLoaded = False
          self._family = None;

      def getFamily(self, current = None):
          self.errorIfUnloaded()
          return self._family

      def setFamily(self, _family, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.family.wrapper is not None:
              if _family is not None:
                  _family = self._field_info.family.wrapper(_family)
          self._family = _family
          pass

      def unloadCoefficient(self, ):
          self._coefficientLoaded = False
          self._coefficient = None;

      def getCoefficient(self, current = None):
          self.errorIfUnloaded()
          return self._coefficient

      def setCoefficient(self, _coefficient, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.coefficient.wrapper is not None:
              if _coefficient is not None:
                  _coefficient = self._field_info.coefficient.wrapper(_coefficient)
          self._coefficient = _coefficient
          pass

      def unloadInputStart(self, ):
          self._inputStartLoaded = False
          self._inputStart = None;

      def getInputStart(self, current = None):
          self.errorIfUnloaded()
          return self._inputStart

      def setInputStart(self, _inputStart, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.inputStart.wrapper is not None:
              if _inputStart is not None:
                  _inputStart = self._field_info.inputStart.wrapper(_inputStart)
          self._inputStart = _inputStart
          pass

      def unloadInputEnd(self, ):
          self._inputEndLoaded = False
          self._inputEnd = None;

      def getInputEnd(self, current = None):
          self.errorIfUnloaded()
          return self._inputEnd

      def setInputEnd(self, _inputEnd, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.inputEnd.wrapper is not None:
              if _inputEnd is not None:
                  _inputEnd = self._field_info.inputEnd.wrapper(_inputEnd)
          self._inputEnd = _inputEnd
          pass

      def unloadActive(self, ):
          self._activeLoaded = False
          self._active = None;

      def getActive(self, current = None):
          self.errorIfUnloaded()
          return self._active

      def setActive(self, _active, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.active.wrapper is not None:
              if _active is not None:
                  _active = self._field_info.active.wrapper(_active)
          self._active = _active
          pass

      def unloadNoiseReduction(self, ):
          self._noiseReductionLoaded = False
          self._noiseReduction = None;

      def getNoiseReduction(self, current = None):
          self.errorIfUnloaded()
          return self._noiseReduction

      def setNoiseReduction(self, _noiseReduction, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.noiseReduction.wrapper is not None:
              if _noiseReduction is not None:
                  _noiseReduction = self._field_info.noiseReduction.wrapper(_noiseReduction)
          self._noiseReduction = _noiseReduction
          pass

      def unloadRed(self, ):
          self._redLoaded = False
          self._red = None;

      def getRed(self, current = None):
          self.errorIfUnloaded()
          return self._red

      def setRed(self, _red, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.red.wrapper is not None:
              if _red is not None:
                  _red = self._field_info.red.wrapper(_red)
          self._red = _red
          pass

      def unloadGreen(self, ):
          self._greenLoaded = False
          self._green = None;

      def getGreen(self, current = None):
          self.errorIfUnloaded()
          return self._green

      def setGreen(self, _green, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.green.wrapper is not None:
              if _green is not None:
                  _green = self._field_info.green.wrapper(_green)
          self._green = _green
          pass

      def unloadBlue(self, ):
          self._blueLoaded = False
          self._blue = None;

      def getBlue(self, current = None):
          self.errorIfUnloaded()
          return self._blue

      def setBlue(self, _blue, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.blue.wrapper is not None:
              if _blue is not None:
                  _blue = self._field_info.blue.wrapper(_blue)
          self._blue = _blue
          pass

      def unloadAlpha(self, ):
          self._alphaLoaded = False
          self._alpha = None;

      def getAlpha(self, current = None):
          self.errorIfUnloaded()
          return self._alpha

      def setAlpha(self, _alpha, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.alpha.wrapper is not None:
              if _alpha is not None:
                  _alpha = self._field_info.alpha.wrapper(_alpha)
          self._alpha = _alpha
          pass

      def unloadLookupTable(self, ):
          self._lookupTableLoaded = False
          self._lookupTable = None;

      def getLookupTable(self, current = None):
          self.errorIfUnloaded()
          return self._lookupTable

      def setLookupTable(self, _lookupTable, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.lookupTable.wrapper is not None:
              if _lookupTable is not None:
                  _lookupTable = self._field_info.lookupTable.wrapper(_lookupTable)
          self._lookupTable = _lookupTable
          pass

      def unloadSpatialDomainEnhancement(self, current = None):
          self._spatialDomainEnhancementLoaded = False
          self._spatialDomainEnhancementSeq = None;

      def _getSpatialDomainEnhancement(self, current = None):
          self.errorIfUnloaded()
          return self._spatialDomainEnhancementSeq

      def _setSpatialDomainEnhancement(self, _spatialDomainEnhancement, current = None, wrap=False):
          self.errorIfUnloaded()
          if wrap and self._field_info.spatialDomainEnhancementSeq.wrapper is not None:
              if _spatialDomainEnhancement is not None:
                  _spatialDomainEnhancement = self._field_info.spatialDomainEnhancementSeq.wrapper(_spatialDomainEnhancement)
          self._spatialDomainEnhancementSeq = _spatialDomainEnhancement
          self.checkUnloadedProperty(_spatialDomainEnhancement,'spatialDomainEnhancementLoaded')

      def isSpatialDomainEnhancementLoaded(self):
          return self._spatialDomainEnhancementLoaded

      def sizeOfSpatialDomainEnhancement(self, current = None):
          self.errorIfUnloaded()
          if not self._spatialDomainEnhancementLoaded: return -1
          return len(self._spatialDomainEnhancementSeq)

      def copySpatialDomainEnhancement(self, current = None):
          self.errorIfUnloaded()
          if not self._spatialDomainEnhancementLoaded: self.throwNullCollectionException("spatialDomainEnhancementSeq")
          return list(self._spatialDomainEnhancementSeq)

      def iterateSpatialDomainEnhancement(self):
          self.errorIfUnloaded()
          if not self._spatialDomainEnhancementLoaded: self.throwNullCollectionException("spatialDomainEnhancementSeq")
          return iter(self._spatialDomainEnhancementSeq)

      def addCodomainMapContext(self, target, current = None):
          self.errorIfUnloaded()
          if not self._spatialDomainEnhancementLoaded: self.throwNullCollectionException("spatialDomainEnhancementSeq")
          self._spatialDomainEnhancementSeq.append( target );
          target.setChannelBinding( self )

      def addAllCodomainMapContextSet(self, targets, current = None):
          self.errorIfUnloaded()
          if  not self._spatialDomainEnhancementLoaded: self.throwNullCollectionException("spatialDomainEnhancementSeq")
          self._spatialDomainEnhancementSeq.extend( targets )
          for target in targets:
              target.setChannelBinding( self )

      def removeCodomainMapContext(self, target, current = None):
          self.errorIfUnloaded()
          if not self._spatialDomainEnhancementLoaded: self.throwNullCollectionException("spatialDomainEnhancementSeq")
          self._spatialDomainEnhancementSeq.remove( target )
          target.setChannelBinding( None )

      def removeAllCodomainMapContextSet(self, targets, current = None):
          self.errorIfUnloaded()
          if not self._spatialDomainEnhancementLoaded: self.throwNullCollectionException("spatialDomainEnhancementSeq")
          for elt in targets:
              elt.setChannelBinding( None )
              self._spatialDomainEnhancementSeq.remove( elt )

      def clearSpatialDomainEnhancement(self, current = None):
          self.errorIfUnloaded()
          if not self._spatialDomainEnhancementLoaded: self.throwNullCollectionException("spatialDomainEnhancementSeq")
          for elt in self._spatialDomainEnhancementSeq:
              elt.setChannelBinding( None )
          self._spatialDomainEnhancementSeq = list()

      def reloadSpatialDomainEnhancement(self, toCopy, current = None):
          self.errorIfUnloaded()
          if self._spatialDomainEnhancementLoaded:
              raise omero.ClientError("Cannot reload active collection: spatialDomainEnhancementSeq")
          if not toCopy:
              raise omero.ClientError("Argument cannot be null")
          if toCopy.getId().getValue() != self.getId().getValue():
             raise omero.ClientError("Argument must have the same id as this instance")
          if toCopy.getDetails().getUpdateEvent().getId().getValue() < self.getDetails().getUpdateEvent().getId().getValue():
             raise omero.ClientError("Argument may not be older than this instance")
          copy = toCopy.copySpatialDomainEnhancement() # May also throw
          for elt in copy:
              elt.setChannelBinding( self )
          self._spatialDomainEnhancementSeq = copy
          toCopy.unloadSpatialDomainEnhancement()
          self._spatialDomainEnhancementLoaded = True

      def getCodomainMapContext(self, index, current = None):
          self.errorIfUnloaded()
          if not self._spatialDomainEnhancementLoaded: self.throwNullCollectionException("spatialDomainEnhancementSeq")
          return self._spatialDomainEnhancementSeq[index]

      def setCodomainMapContext(self, index, element, current = None, wrap=False):
          self.errorIfUnloaded()
          if not self._spatialDomainEnhancementLoaded: self.throwNullCollectionException("spatialDomainEnhancementSeq")
          old = self._spatialDomainEnhancementSeq[index]
          if wrap and self._field_info.spatialDomainEnhancementSeq.wrapper is not None:
              if element is not None:
                  element = self._field_info.spatialDomainEnhancementSeq.wrapper(_spatialDomainEnhancement)
          self._spatialDomainEnhancementSeq[index] =  element
          if element is not None and element.isLoaded():
              element.setChannelBinding( self )
          return old

      def getPrimaryCodomainMapContext(self, current = None):
          self.errorIfUnloaded()
          if not self._spatialDomainEnhancementLoaded: self.throwNullCollectionException("spatialDomainEnhancementSeq")
          return self._spatialDomainEnhancementSeq[0]

      def setPrimaryCodomainMapContext(self, element, current = None):
          self.errorIfUnloaded()
          if not self._spatialDomainEnhancementLoaded: self.throwNullCollectionException("spatialDomainEnhancementSeq")
          index = self._spatialDomainEnhancementSeq.index(element)
          old = self._spatialDomainEnhancementSeq[0]
          self._spatialDomainEnhancementSeq[index] = old
          self._spatialDomainEnhancementSeq[0] = element
          return old


      def ice_postUnmarshal(self):
          """
          Provides additional initialization once all data loaded
          """
          pass # Currently unused


      def ice_preMarshal(self):
          """
          Provides additional validation before data is sent
          """
          pass # Currently unused

      def __getattr__(self, name):
          """
          Reroutes all access to object.field through object.getField() or object.isField()
          """
          if "_" in name:  # Ice disallows underscores, so these should be treated normally.
              return object.__getattribute__(self, name)
          field  = "_" + name
          capitalized = name[0].capitalize() + name[1:]
          getter = "get" + capitalized
          questn = "is" + capitalized
          try:
              self.__dict__[field]
              if hasattr(self, getter):
                  method = getattr(self, getter)
                  return method()
              elif hasattr(self, questn):
                  method = getattr(self, questn)
                  return method()
          except:
              pass
          raise AttributeError("'%s' object has no attribute '%s' or '%s'" % (self.__class__.__name__, getter, questn))

      def __setattr__(self, name, value):
          """
          Reroutes all access to object.field through object.getField(), with the caveat
          that all sets on variables starting with "_" are permitted directly.
          """
          if name.startswith("_"):
              self.__dict__[name] = value
              return
          else:
              field  = "_" + name
              setter = "set" + name[0].capitalize() + name[1:]
              if hasattr(self, field) and hasattr(self, setter):
                  method = getattr(self, setter)
                  return method(value)
          raise AttributeError("'%s' object has no attribute '%s'" % (self.__class__.__name__, setter))

_omero_model.ChannelBindingI = ChannelBindingI
